#ifndef AMBIENTEH
#define AMBIENTEH

// INCLUDE
#include <cmath>

/*****************************************************//**
* \class	Ambiente
* \brief 	Definição do ambiente
* \author	Cyro Albuquerque Neto
* \date		Criação: 18/08/2008
* \date		Revisão: 18/08/2008
*
* Define o ambiente
*********************************************************/

class Ambiente {

////////////////////////////// PUBLIC ////////////////////////////////////////
public:
	Ambiente() {}
	void SetPropriedade(double tOper=30.0, double uRel=0.5, double pBar=101325.0) {
		tempOperativa = tOper;
		umidadeRel = uRel;
		pressBar = pBar;
	}
	double TempOperativa() const { return tempOperativa; }
	void SetTempOperativa(double num) { tempOperativa = num; }
	double UmidadeRel() const { return umidadeRel; }
	void SetUmidadeRel(double num) { umidadeRel = num; }
	double PressBar() const { return pressBar; }
	void SetPressBar(double num) { pressBar = num; }

	/*********************************************************************//**
	* \brief Calcula pressão de vapor d'água no ar [Pa]
	* \param temperatura [oC]
	*
	* Referência ASHRAE (2005)
	*************************************************************************/
	double PressVaporAgua(double temperatura);
	double EntalpiaVapAgua(double tempAr);
	double CalorEspAr(double tempAr);
	double UmidadeAbs(double tempAr, double umidRel, double pBar);
	double DensidadeAr(double tempAr, double pBar);

////////////////////////////// PRIVATE ///////////////////////////////////////
private:
	double tempOperativa;
	double umidadeRel;
	double pressBar;
};

#endif
