#include "celula.h"

///////////////////////////////// PUBLIC /////////////////////////////////////

/*****************************************************************************
 * Construtor
 ****************************************************************************/
Celula::Celula(	IntPos				nDim, 
				SistCoord			sistCoord, 
				vector<Camada*>&	cam, 
				VetInt&				jCel, 
				CompTecido*			cTec, 
				Ponto<double>		tamExt) 
	: temperatura(34.0), 
	  fonteC(0.0), 
	  fonteP(0.0)
{ 
	//pressao.O2 = 33.0;
	//concentracao.O2 = 0.2;
	//pressao.CO2 = 40.0;
	//concentracao.CO2 = 0.5;
	compTec = cTec;

	// Tamanho da celula
	//if (sistCoord==CARTESIANO) tamanho.SetPonto(1.0,1.0,1.0);
	//else if (sistCoord==CILINDRICO) tamanho.SetPonto(1.0,2.0*M_PI,1.0);
	//else if (sistCoord==CIL_RZ) tamanho.SetPonto(1.0,1.0,2.0*M_PI);
	tamanho = tamExt;
	for (IntPos i=0; i<nDim; i++)
		tamanho.SetPtCoord(i,cam[i]->TamCelula());

	//Ponto central da celula
	for (IntPos i=0; i<nDim; i++)
		centro.SetPtCoord(i,cam[i]->Posicao().ini 
						  +(jCel[i]+0.5)*cam[i]->TamCelula());

	//Volume da celula
	CalcVolume(nDim,sistCoord);
}

/*****************************************************************
 * Novo vizinho
 ****************************************************************/
void Celula::NovoVizinho(Celula *vCel, Face *vFace, Sentido vSent) 
{
	ElVizinho vizAux = {vCel,vFace,vSent,0.0};
	vizinho.push_back(vizAux);
}

/*****************************************************************
 * Face dada o sentido
 ****************************************************************/
Face* Celula::FaceSentido(Sentido sentFace)
{
	IntPos i=0;
	for (; i<vizinho.size(); ++i)
		if (vizinho[i].sentido==sentFace) break;
	return vizinho[i].face;
}

/*****************************************************************
 * Celula vizinha dada o sentido
 ****************************************************************/
Celula* Celula::CelVizSentido(Sentido sentFace)
{
	IntPos i=0;
	for (; i<vizinho.size(); ++i)
		if (vizinho[i].sentido==sentFace) break;
	return vizinho[i].cel;
}

/*****************************************************************
 * Verifica se a celula e central, nao tendo a face W
 ****************************************************************/
bool Celula::EhCentro()
{
	bool centro=true;
	IntPos i=0;
	for (; i<vizinho.size(); ++i)
		if (vizinho[i].sentido==W) centro=false;
	return centro;
}


///////////////////////////////// PRIVATE /////////////////////////////////////

/*****************************************************************
 * Calcula o volume de uma celula. 
 ****************************************************************/
void Celula::CalcVolume(IntPos /*nDim*/, SistCoord sistCoord)
{
	if (sistCoord==CARTESIANO) 
		volume = tamanho.C1()*tamanho.C2()*tamanho.C3();
	
	else if (sistCoord==CILINDRICO) {
		double vRaio = pow(centro.C1() + tamanho.C1()/2.0,2.0)
				- pow(centro.C1() - tamanho.C1()/2.0,2.0);
		volume = vRaio*tamanho.C2()*tamanho.C3()/2.0;
	}
	else if (sistCoord==CIL_RZ) {
		double vRaio = pow(centro.C1() + tamanho.C1()/2.0,2.0)
				- pow(centro.C1() - tamanho.C1()/2.0,2.0);
		volume = vRaio*tamanho.C3()*tamanho.C2()/2.0;
	}
}

/*****************************************************************
 * Direcao oposta
 ****************************************************************/
Sentido Celula::SentidoOposto(Sentido sent) 
{
	if (sent==W) return E;
	else if (sent==E) return W;
	else if (sent==S) return N;
	else if (sent==N) return S;
	else if (sent==F) return B;
	else /*if (sent==B)*/ return F;
}

/*****************************************************************
 * Celula oposta
 ****************************************************************/
IntPos Celula::VizOposto(IntPos val) 
{
	IntPos i=0;
	for (; i<vizinho.size(); ++i)
		if (vizinho[i].sentido==SentidoOposto(vizinho[val].sentido)) break;
	return i;
}


/*****************************************************************
 * Corrige o metabolismo em funcao da temperatura
 ****************************************************************/
double Celula::CorrMetabolismo(double metabolismo)
{
return metabolismo;
//return metabolismo*pow(2.0,(temperatura-compTec->TempBasal())/10.0);
}

