/*************************************************************************//**
* \class	Corpo
* \brief 	Representao de todo o corpo humano
* \author	Cyro Albuquerque Neto
* \date		Criao: 18/08/2008
* \date		Reviso: 03/06/2009
*
* Contm os segmento.
*****************************************************************************/

#ifndef CORPOH
#define CORPOH

// INCLUDES DO SISTEMA
#include <iostream>
#include <fstream>
#include <vector>
#include <cstdlib>
#include <string>

// INCLUDES LOCAIS
#include "definicao.h"
#include "matriz.h"
#include "celula.h"
#include "face.h"
#include "segmento.h"
#include "proptecido.h"
#include "compartimento.h"
#include "transportesangue.h"

using namespace std;

class Corpo 
{
public:

// CONSTRUTORES E DESTRUTORES

	/** Construtor
	 */
	Corpo();

	/** Destrutor
	 */
	~Corpo() {}

	
// OPERAES

	/** Calcula regime permanente
	 */
	//void CalculaRegPermanente();

	/** Calcula regime transitorio
	 */
	//void CalculaRegTransitorio();
	void Calcula();

	
private:

// ATRIBUTOS

	/// Propriedades dos tecidos
	// No caso dos musculos, existem para cada membro;
	// porque suas propriedades mudam em funcao da atividade fisica.
	PropTecido pele,
			   gordura,
			   osso, 
			   cerebro, 
			   viscera, 
			   pulmao, 
			   coracao,
			   muscCabeca,	
			   muscPescoco, 	
			   muscTronco, 
			   muscAnteBraco, 
			   muscBraco, 
			   muscMao, 
			   muscCoxa, 
			   muscPerna, 
			   muscPe, 
			   muscBasal;

	/// Propriedades do ambiente
	Ambiente ambiente;

	/// Segmentos
	Segmento cabeca, 
			 pescoco, 
			 tronco, 
			 antebracoEsq, 
			 bracoEsq, 
			 maoEsq, 
			 antebracoDir,
			 bracoDir,
			 maoDir,
			 coxaEsq, 
			 pernaEsq, 
			 peEsq,
			 coxaDir,
			 pernaDir,
	         peDir;


	/// Vetor com endereco dos segmentos
	vector<Segmento*> lSegmento;

	vector<Segmento*> lSegmentoSaida;

	/// Fator de correo da resistncia  temperatura entre pele e ambiente
	double corrResistTemp;

	/// Fator de correo da resistncia  evaporao entre pele e ambiente
	double corrResistEvap;

// INICIALIZAES

	/** Leitura do arquivo com propriedade dos tecidos
	 */
	void LePropriedadeTecido();

	/** Funo para inicializar o segmento da cabea
	 */
	void CriaSegmento(Segmento& seg, const char* nomeArq);

	void CriaCabeca();

	void CriaPescoco();
	void CriaTronco();
	void CriaBraco();
	void CriaAntebraco();
	void CriaMao();
	void CriaCoxa();
	void CriaPerna();
	void CriaPe();

// OPERAES

	/** Ajusta as vazes de sangue entre os segmentos
	 */
	void AjustaVazao();

	/** Ajusta temperatura do sangue entre os segmentos
	 */
	void AjustaTemperatura();

	/** Ajusta concentrao do sangue entre os segmentos
	 */
	void AjustaConcentracao();

	/** Ajusta o calor trocado na respirao
	 */
	void AjustaCalorRespiracao();

	/** Calcula temperatura mdia dos tecidos
	 */
	void CalcTempMediaTecido();

	TipoTecido RetornaTipoTecido(IntPos num);

	double Calafrio(double tempPele, double tempInterna, double calorSup);
	double Sudorese(double tempPeleMedia, double tempPeleLocal, double tempInterna);
	double VazaoPele(double tempPele, double tempInterna);

	void SaidaPolar(int iz, const char* nome, Segmento& segmento);
	void SaidaXZ(int iz, const char* nome, Segmento& segmento);
	void SaidaXY(int iz, const char* nome, Segmento& segmento);
	void SaidaLinha(const char* nome, Segmento& segmento);
	//void SaidaCompTec(int iz, const char* nome, Segmento& segmento);

	void inicializaSaidas(vector<ofstream*>& saidaTrans);

};

#endif
