//Definicoes
//Autor: Cyro Albuquerque Neto
//Data: 12/08/2008

#ifndef DEFINICAOH
#define DEFINICAOH

#include <vector>

/// Preciso usada nos mtodos numricos
double const PRECISAO = 1.0e-6;

/// Nmero de segmentos
unsigned int const NUM_SEGMENTO = 15;

/// Nmero de gases
unsigned int const NUM_GAS = 2;

/// Nmero do gs
unsigned int const GO2 = 0;
unsigned int const GCO2 = 1;

/// Tipos
typedef unsigned int IntPos;
typedef std::vector<unsigned int> VetInt;

/// Enumeraes
enum SistCoord {CARTESIANO=0,CILINDRICO,CIL_RZ,ESFERICO};
enum Sentido {W=0,E,S,N,F,B};
enum Direcao {WE=0,SN,FB};

/// Saida
#define TAB "\t"

/** Define a geometria do segmento, podendo ter coordenadas cilndricas
 * ou cartesianas
 * RADIAL: uma dimenso, coordenada $r$
 * AXIAL: duas dimenses, coordenadas $r$ e $z$
 * POLAR: duas dimenses, coordenadas $r$ e $\theta$
 * RETANGULAR: duas dimenses, coordenadas $x$ e $y$
 */
enum GeometriaSegmento {RADIAL=0,AXIAL,POLAR,RETANGULAR};
//usar isso ao invs de sistCoordenadas e nDim

template <typename T> struct Limite { T ini, fim; };

template<class T> class Gas
{
    public:
        static const IntPos nGas = 2;
        Gas() {}
        Gas(T val1, T val2) { o2=val1; co2=val2; }
        T O2() const { return o2; }
        T CO2() const { return co2; }
        void SetGas(T val1, T val2) { o2=val1; co2=val2; }
        void SetGas(Gas<T> val) { o2=val.O2(); co2=val.CO2(); }
        void SetGas(IntPos i, T val) { 
            if (i==0) o2=val;
            else co2=val;
        }
        void SetO2(T val) { o2=val; }
        void SetCO2(T val) { co2=val; }
        T Em(IntPos num) const {
            if (num==0) return o2;
            else return co2;
        }
        /// Atribui valor
        T& operator() (IntPos num) { 
            if (num==0) return o2;
            else return co2;
        }
        /// Retorna valor
        T operator() (IntPos num) const {
            if (num==0) return o2;
            else return co2;
        }
        /// Multiplica por um valor
        void MultiplicaPor(double val) {
            o2 *= val;
            co2 *= val;
        }
        /// Divide por um valor
        void DividePor(double val) {
            o2 /= val;
            co2 /= val;
        }


    private:
        T o2,co2;
};

#endif
