#ifndef FACEH
#define FACEH

/*****************************************************************
Classe:     Face, FaceInterna e FaceExterna
Descricao:  Definicao do elemento face
Arquivo:    Face.h
Autor:      Cyro Albuquerque Neto
Criacao:    18/08/2008
Revisao:    18/08/2008
******************************************************************/

#include "definicao.h"
#include "ponto.h"
#include "celula.h"

enum TipoElemento {FACE_INT,FACE_EXT};

class Celula;

class Face {
protected:
    TipoElemento tipo;
    Ponto<double> centro;
    double dCelula, condTerm;
    Direcao direcao;
    Sentido sentido;
public:
    //Construtor
    Face() : dCelula(0.0), condTerm(0.0) {}
    virtual ~Face() {}

    //Tipo do elemento
    void SetTipo(TipoElemento valor) { tipo=valor; }
    TipoElemento Tipo() const { return tipo; }
    
    //Ponto no centro do elemento
    void SetCentro(double val1, double val2=0.0, double val3=0.0) 
        { centro.SetPonto(val1,val2,val3); }
    void SetCentroCoord(int k, double val) 
        { centro.SetPtCoord(k,val); }
    Ponto<double> Centro() const {return centro;}
    
    //Distancia entre centros das celulas, no caso de face interna;
    //ou entre centro da celula e face, no caso de face externa
    void SetDCelula(double valor) { dCelula=valor; }
    double DCelula() const { return dCelula; }
    
    //Direcao
    void SetDirecao(Direcao valor) { direcao=valor; }
    Direcao GetDirecao() const { return direcao; }
    
    //Sentido
    void SetSentido(Sentido valor) { sentido=valor; }
    Sentido GetSentido() const { return sentido; }

    //Celulas vizinhas
    virtual void SetCelViz1(Celula *valor) = 0;
    virtual void SetCelViz2(Celula*) {}
    virtual Celula* CelViz1() = 0;
    virtual Celula* CelViz2() { return 0; }
    
    //Condutividade termica
    double CondTerm() const { return condTerm; }
    void SetCondTerm(double valor) { condTerm=valor; }
    virtual void CalcCondTerm() = 0; 
    
    //Temperatura
    virtual double Temperatura() const { return 0; }
    virtual void SetTemperatura(double) { }

    virtual double AreaSuperficial(SistCoord) { return 0; }
};

class FaceInterna : public Face {
    Celula *celViz1, *celViz2;

public:
    //Construtor
    FaceInterna(Direcao vDir=WE) { 
        Face::tipo=FACE_INT;
        Face::direcao=vDir;   
    } 
    ~FaceInterna() {}

    //Celulas vizinhas
    void SetCelViz1(Celula *valor) { celViz1=valor; }
    void SetCelViz2(Celula *valor) { celViz2=valor; } 
    Celula* CelViz1() { return celViz1; } //anterior
    Celula* CelViz2() { return celViz2; } //posterior

    //Calculo da condutividade termica na face,
    //funcao das condutividades vizinhas
    void CalcCondTerm() { 
        double fe = celViz1->Tamanho().PtCoord(Face::direcao)/(2.0*dCelula);
        Face::condTerm = 
            1.0/(fe/celViz1->CondTerm() + (1-fe)/celViz2->CondTerm());
    }
};

class FaceExterna : public Face {
    Celula *celViz;
    double temperatura;

public:
    //Construtor
    FaceExterna(Sentido vSent=W) : temperatura(34.0) { 
        Face::tipo=FACE_EXT;
        Face::sentido=vSent;   
    }
    ~FaceExterna() {}

    double Temperatura() const { return temperatura; }
    void SetTemperatura(double valor) { temperatura=valor; }

    //Celulas vizinhas
    void SetCelViz1(Celula *valor) { celViz=valor; }
    Celula* CelViz1() { return celViz; }
    
    //Calculo da condutividade termica
    void CalcCondTerm() 
        { Face::condTerm = celViz->CondTerm(); }

    double AreaSuperficial(SistCoord sistCoordenada) {
        if (sistCoordenada==CARTESIANO) {
            if (direcao==WE) {
                return celViz->Tamanho().C2()*celViz->Tamanho().C3();
            }
            else if (direcao==SN) {
                return celViz->Tamanho().C3()*celViz->Tamanho().C1();
            }
            else /*if (direcao==FB)*/ {
                return celViz->Tamanho().C1()*celViz->Tamanho().C2();
            }
        }
        else {
            if (direcao==WE) 
                return Face::centro.C1()*celViz->Tamanho().C2()
                    *celViz->Tamanho().C3();
            else return 0;  
        }
    }       

};

#endif
