// Funcoes membro da classe ListaCelula

#include "listacelula.h"

/*****************************************************************
* Construtor
******************************************************************/
ListaCelula::ListaCelula()
{
}

/*****************************************************************
* Destrutor
******************************************************************/
ListaCelula::~ListaCelula()
{
	for (IntPos i=0; i<celulas.DimVet(); ++i) delete celulas(i);
	celulas.Vet().clear();
}

/***************************************************************** 
* Gera lista de celulas
*****************************************************************/
void ListaCelula::GeraListaCelula(IntPos nDim, SistCoord sistCoord,
								  vector<ListaCamada>& lCamada, 
							      Matriz<CompTecido*>& vCompTec,
								  Ponto<double> tamExt)
{
	//Calcula numero de celulas em cada coordenada 
	//e ajusta tamanho da matriz de celulas
	for (IntPos i=0; i<nDim; ++i) 
		celulas.SetDimCoord(i,lCamada.at(i).NCelula());

	//Gera as celulas
	VetInt iC(nDim,0),				//numero da celula em cada direcao 
		   iCam(nDim,0),			//numero da camada em cada direcao 
		   iCel(nDim,0);			//numero da celula na camada em cada direcao
	vector<Camada*> vCamada(nDim);	//camadas de uma celula em todas direcoes
	//Varre camadas e celulas de cada camada, em cada direcao
	for (iCam[0]=0; iCam[0]<lCamada[0].Tam(); ++iCam[0]) {
		for (iCel[0]=0; iCel[0]<lCamada[0].Em(iCam[0])->NCelula(); ++iCel[0]) {
			iC[0] = lCamada[0].Em(iCam[0])->CelLim().ini+iCel[0];
			vCamada[0] = lCamada[0].Em(iCam[0]);
			if (nDim==1)
				celulas(iC) = new Celula(nDim,sistCoord,vCamada,iCel,vCompTec(iCam[0]),tamExt);
			else {
				for (iCam[1]=0; iCam[1]<lCamada[1].Tam(); ++iCam[1]) {					
					for (iCel[1]=0; iCel[1]<lCamada[1].Em(iCam[1])->NCelula(); ++iCel[1]) {
						iC[1] = lCamada[1].Em(iCam[1])->CelLim().ini+iCel[1];
						vCamada[1] = lCamada[1].Em(iCam[1]);
						if (nDim==2)
							celulas(iC) = new Celula(nDim,sistCoord,vCamada,iCel,vCompTec(iCam[0],iCam[1]),tamExt);
						else {
							for (iCam[2]=0; iCam[2]<lCamada[2].Tam(); ++iCam[2]) {
								for (iCel[2]=0; iCel[2]<lCamada[2].Em(iCam[2])->NCelula(); ++iCel[2]) {
									iC[2] = lCamada[2].Em(iCam[2])->CelLim().ini+iCel[2];
									vCamada[2] = lCamada[2].Em(iCam[2]);
									celulas(iC) = new Celula(nDim,sistCoord,vCamada,iCel,vCompTec(iCam[0],iCam[1],iCam[2]),tamExt);
								}
							}
						}
					}
				}
			}
		}
	}
	//Teste soma dos volumes
	/*double volAux = 0.0;
	for (IntPos i=0; i<celulas.DimVet(); ++i) volAux+=celulas(i)->Volume();
	cout << "Volume total somando celulas: " << volAux << endl;
	cout << "Volume total calculado: " << M_PI*pow(tamanhoExterno.C1(),2.0)*tamanhoExterno.C3() << endl;*/
}

//-----------------------------------------------------------------------------
double ListaCelula::CalcNumFace(IntPos nDim, SistCoord sistCoord) 
{
	//Numero de faces
	//---depende do sistema de coordenadas
	if (sistCoord==CARTESIANO) {
		if (nDim==1) return celulas.Dim1()+1;
		else if (nDim==2) return celulas.Dim1()*(celulas.Dim2()+1) 
							 + celulas.Dim2()*(celulas.Dim1()+1);
		else return celulas.Dim1()*celulas.Dim2()*(celulas.Dim3()+1) 
							 + celulas.Dim3()*celulas.Dim1()*(celulas.Dim2()+1)
							 + celulas.Dim2()*celulas.Dim3()*(celulas.Dim1()+1);
	}
	else if (sistCoord==CILINDRICO) {
		if (nDim==1) return celulas.Dim1()+1;
		else if (nDim==2) return 2*celulas.Dim1()*celulas.Dim2();
		else return 2*celulas.Dim1()*celulas.Dim2()*celulas.Dim3()
					+ celulas.Dim1()*celulas.Dim2()*(celulas.Dim3()+1);
	}
	else if (sistCoord==CIL_RZ) {
		if (nDim==1) return celulas.Dim1()+1;
		else if (nDim==2) return celulas.Dim1()*(celulas.Dim2()+1) 
							 + celulas.Dim2()*(celulas.Dim1()+1);
	}
	else return 0;
	return 0;
}

//-----------------------------------------------------------------------------
//---- Depende sistema coordenada
//---- Se coordenada for teta e ultima celula, celPos e a celula inicial
VetInt ListaCelula::AchaCelPos(SistCoord sistCoord, VetInt cAnt, IntPos coord)
{
	VetInt cPos(cAnt);
	//caso teta e ultima posicao, cilindrico
	if (coord==1 && sistCoord==CILINDRICO && celulas.Fim(cAnt,1)) cPos[1]=0;
	else cPos[coord]+=1;
	return cPos;
}

/*****************************************************************************
* Salva as temperaturas das celulas para uma matriz
*****************************************************************************/
void ListaCelula::AtualizaMatrizTemperatura(Matriz<double> &matTemp)
{
	for (IntPos i=0; i<celulas.DimVet(); ++i) {
		matTemp(i) = celulas(i)->Temperatura();
	}
}

/*****************************************************************************
* Salva as temperaturas das celulas para uma matriz
*****************************************************************************/
void ListaCelula::AtualizaTempAntiga()
{
	for (IntPos i=0; i<celulas.DimVet(); ++i) {
		celulas(i)->SetTemperaturaAnt(celulas(i)->Temperatura());
	}
}
