#ifndef LISTACELULAH
#define LISTACELULAH

#include <cmath>
#include <iostream>
#include <vector>

#include "matriz.h"
#include "definicao.h"
#include "camada.h"
#include "celula.h"
#include "ponto.h"
#include "compartimento.h"

using namespace std;

/*****************************************************//**
* \class	ListaCelula 
* \brief	Lista de celulas
* \author	Cyro Albuquerque Neto
* \date		Criacao: 10/10/2008
* \date		Revisao: 10/10/2008
*********************************************************/
class Celula;
class Camada;

class ListaCelula {

public:
	/**
	 * Construtor
	 */
	ListaCelula();
	//ListaCelula(IntPos dimensao, SistCoord sistCoordenada,
	//		    vector<ListaCamada>& listaCamada, 
	//		    Matriz<PropTerm*>& propTermica);

	/**
	* Destrutor
	* \brief Apaga lista de celulas
	*/
	~ListaCelula();

	/**
	 * Numero de celulas em uma direcao
	 */
	IntPos Dim(IntPos coord) { return celulas.DimCoord(coord); } 
	IntPos DimVet() { return celulas.DimVet(); }

	bool Ini(VetInt val, IntPos coord) { return celulas.Ini(val,coord); }
	bool Fim(VetInt val, IntPos coord) { return celulas.Fim(val,coord); }


	Celula* Em(IntPos valor) { return celulas(valor); }
	Celula* Em(IntPos val1, IntPos val2) 
		{ return celulas(val1,val2); }
	Celula* Em(IntPos val1, IntPos val2, IntPos val3) 
		{ return celulas(val1,val2,val3); }
	Celula* Em(VetInt valor) { return celulas(valor); }
	Matriz<Celula*>* Vet() { return &celulas; }

	double CalcNumFace(IntPos nDim, SistCoord sistCoord); 

	VetInt AchaCelPos(SistCoord sistCoord, VetInt cAnt, IntPos coord);

	void GeraListaCelula(IntPos, SistCoord, vector<ListaCamada>&, Matriz<CompTecido*>&, Ponto<double> tamExt);

	void AtualizaMatrizTemperatura(Matriz<double> &matTemp);
	void AtualizaTempAntiga();

private:
	/// Lista de celulas 
	Matriz<Celula*> celulas;

	/**
	* \brief Gera a lista de celulas
	* \param lCamada vetor com lista de camadas em cada direcao
	* \param vPropTerm matriz com os ponteiros das propriedades
	* termicas, entre os limites das camadas.
	*
	* Funcao principal da geracao da lista das celulas. 
	* Atualiza o tamanho externo do segmento.
	* Calcula numero de celulas em cada coordenada (chama NCelDir).
	* Varre as camadas e as celulas de cada camada, em todas as direcoes; 
	* chamando funcao para criar as celulas (CriaCelula).
	*/

};

#endif
