#ifndef MATRIZH
#define MATRIZH

#include <iostream>
#include <cstdlib>

typedef unsigned int IntP;

using namespace std;

/***********************************************************//**
* \class	Matriz 
* \brief	Representação de uma matriz com 1, 2 ou 3 dimensões
* \author	Cyro Albuquerque Neto
* \date		Criacao: 28/08/2008
* \date		Revisao: 25/09/2008
*
* A Matriz é formada por um vetor, usando o vector (STL).
* Tem informações sobre suas dimensões, que fazem com que
* o elemento correto do vetor seja acessado.
***************************************************************/

template<class T> class Matriz {
	/// Dimensões
	IntP dimens1, dimens2, dimens3, dimensVet;
	/// Vetor
	vector<T> vet;
	IntP CalcDimVet() { return dimens1*dimens2*dimens3; }
	/// Converte indice da matriz para vetor
	IntP MatVet(IntP v1, IntP v2, IntP v3=0) 
		{ return v1+v2*dimens1+v3*dimens1*dimens2; }
	/// Verifica se dimensoes respeitam tamanho da matriz
	bool ChecaMatriz(IntP v1, IntP v2, IntP v3=0) {
		if (v1<dimens1 && v2<dimens2 && v3<dimens3)
			return true;
		else return false;
	}
	/*bool ChecaVetor(IntP val) {
		if (val<dimensVet) return true;
		else return false;
	}*/
	void Erro() {
		cout << "Erro - fora das dimensoes da matriz" << endl;
		exit(1);
	}	
public:
	/// Construtor - sem entrar dimensoes
	Matriz() : dimens1(1), dimens2(1), dimens3(1), dimensVet(1) {};
	/// Construtor - entrada das dimensoes
	Matriz(IntP val1, IntP val2=1, IntP val3=1) : 
		dimens1(val1), dimens2(val2), dimens3(val3) {
		dimensVet = CalcDimVet();
		vet.resize(dimensVet);
	}
	/// Construtor - entrada de dimensoes e valor inicial 
	Matriz(IntP val1, IntP val2, IntP val3, T valor) :
		dimens1(val1), dimens2(val2), dimens3(val3) {
		dimensVet = CalcDimVet();
		vet.resize(dimensVet,valor);
	}
	/// Atribui valor as dimensoes
	void SetDimensao(IntP val1, IntP val2=1, IntP val3=1) {
		dimens1 = val1;
	   	dimens2 = val2;
	   	dimens3 = val3;
		dimensVet = CalcDimVet();
		vet.resize(dimensVet);
	}
	/// Atribui valor as dimensoes e valor inicial
	void SetDimensao(IntP val1, IntP val2, IntP val3, T valor) {
		dimens1 = val1;
	   	dimens2 = val2;
	   	dimens3 = val3;
		dimensVet = CalcDimVet();
		vet.resize(dimensVet,valor);
	}
	void SetDimCoord(IntP k, IntP val) {
		if (k==0) dimens1 = val;
		else if (k==1) dimens2 = val;
		else if (k==2) dimens3 = val;
		dimensVet = CalcDimVet();
		vet.resize(dimensVet);
	}
	/// Atribui valor para o elemento i
	T& operator() (IntP val) { 
		//if (!ChecaVetor(val)) Erro(); 
		return vet[val];
	}
	/// Retorna valor do elemento i
	T operator() (IntP val) const { 
		//if (!ChecaVetor(val)) Erro(); 
		return vet[val];
	}
	/// Atribui valor para o elemento i,j,k
	T& operator() (IntP v1, IntP v2, IntP v3=0) {
		if (!ChecaMatriz(v1,v2,v3)) Erro(); 
		return vet[MatVet(v1,v2,v3)];
	}
	/// Retorna valor do elemento i,j,k
	T operator() (IntP v1, IntP v2, IntP v3=0) const { 	
		if (!ChecaMatriz(v1,v2,v3)) Erro(); 
		return vet[MatVet(v1,v2,v3)];
	}
	/// Atribui valor para elemento com entrada vector<int>
	T& operator() (vector<IntP>& val) {
		if (val.size()==1) {
			//if (!ChecaVetor(val[0])) Erro();
			return vet[val[0]];
		}
		else {
			IntP val3=0;
			if (val.size()==3) val3=val[2];
			if (!ChecaMatriz(val[0],val[1],val3)) Erro();
			return vet[MatVet(val[0],val[1],val3)];
		}
	}
	/// Retorna valor do elemento com entrada vector<int>
	T operator() (vector<IntP> val) const {
		if (val.size()==1) {
			//if (!ChecaVetor(val[0])) Erro();
			return vet[val[0]];
		}
		else {
			IntP val3=0;
			if (val.size()==3) val3=val[2];
			if (!ChecaMatriz(val[0],val[1],val3)) Erro();
			return vet[MatVet(val[0],val[1],val3)];
		}
	}
	/// Retorna a matriz como tipo vector 
	vector<T> Vet() { return vet; }
	/// Dimensao
	IntP Dim1() { return dimens1; }
	IntP Dim2() { return dimens2; }
	IntP Dim3() { return dimens3; }
	IntP DimVet() { return dimensVet; }
	IntP DimCoord(IntP k) {
		if (k==0) return dimens1;
		else if (k==1) return dimens2;
		else return dimens3;
	}
	/// Verifica se o elemento é inicial para uma coordenada
	bool Ini(IntP val, IntP coord) { 
		if (val==0) return true;
		else return false;
	}
	bool Ini(vector<IntP>& val, IntP coord) { 
		if (val[coord]==0) return true;
		else return false;
	}
	/// Verifica se o elemento é final para uma coordenada
	bool Fim(IntP val, IntP coord) { 
		if (val==DimCoord(coord)-1) return true;
		else return false;
	}
	bool Fim(vector<IntP>& val, IntP coord) { 
		if (val[coord]==DimCoord(coord)-1) return true;
		else return false;
	}
};

#endif
