// Funcoes membro da classe Segmento

#include "segmento.h"
#include "boost/thread/thread.hpp"
#include <boost/thread/barrier.hpp>
#include <ctime>

extern double tempoTotPulmao;
extern double tempoTotForPulmao;
extern double tempoTotAtConc;
extern double totExecCompPulm;
//extern double rpc;
//extern double nrpc;

//Gas<double> Segmento::presAlveolar;
//Gas<double> Segmento::difPulmao;
//double Segmento::consumoO2Total;
//double Segmento::tempInternaMedia;
//double Segmento::tempPeleMedia;
double Segmento::ventilacao;
CompPulmao Segmento::compPulmao;
Ambiente *Segmento::amb;
Gas<double> Segmento::presInspirada;

extern bool   regimePermanente;
extern double passoTempo;
extern double tempoSimulacao;
extern double relaxaPres;
extern double geracaoCalorMusculo;

/*****************************************************************************
* Forma do segmento
*****************************************************************************/
void Segmento::SetForma(IntPos dimensao, SistCoord sistCoordenada)
{
    nDim=dimensao;
    sistCoord=sistCoordenada;
}

/*****************************************************************************
* Inicializa
*****************************************************************************/
void Segmento::Inicializa(vector<ListaCamada>& listaCamada, 
        Matriz<CompTecido*>& compTec) 
{
    lCelula.GeraListaCelula(nDim,sistCoord,listaCamada,compTec,tamanhoExterno);
    lFace.GeraListaFace(nDim,sistCoord,lCelula,tamanhoExterno);
    //associa numero dos compartimentos teciduais
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].SetNumComp(i);
    }
    //calcula volume dos compartimentos teciduais
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].ZeraVolTotal();
    }
    for (IntPos i=0; i<lCelula.DimVet(); ++i) {
        lCompTecido[lCelula.Em(i)->NumCompCel()].SomaVolTotal(lCelula.Em(i)->Volume());
    }
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].AtualizaVolume();
    }

    ehTronco=false;
}


/*****************************************************************************
* Calcula geracao de calor no segmento
*****************************************************************************/
double Segmento::GerCalor() 
{
    double geracao = 0.0;
    for (IntPos i=0; i<lCelula.DimVet(); ++i)
        geracao += lCelula.Em(i)->GerCalor()*lCelula.Em(i)->Volume();
    
    //for (IntPos i=0; i<lCompTecido.size(); ++i)
    //  geracao += lCompTecido[i].GerCalor();
    return geracao;
}

/*****************************************************************************
* Calcula vazao de sangue nos pequenos vasos
*****************************************************************************/
double Segmento::VazSanguePeqVas() 
{
    vazSanguePeqVas = 0.0;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()!=PULMAO)
            vazSanguePeqVas += lCompTecido[i].VazSangue();
    }
    return vazSanguePeqVas;
}

/*****************************************************************************
* Calcula volume de tecido
*****************************************************************************/
double Segmento::VolumeTecido(TipoTecido tipo) 
{
    double volTec = 0.0;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==tipo)
            volTec += lCompTecido[i].VolTotal();
    }
    return volTec;
}

/*****************************************************************************
* Temperatura de um tipo de tecido
*****************************************************************************/
double Segmento::TempCompTecido(TipoTecido tipo)
{
    double tempTec = 0.0, volTec = 0.0;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==tipo) {
            volTec += lCompTecido[i].VolTotal();
            tempTec += lCompTecido[i].Temperatura()*lCompTecido[i].VolTotal();
        }
    }
    if (volTec!=0.0) {
        tempTec/=volTec;
        return tempTec;
    }
    else return 0.0;
}

/*****************************************************************************
* Calcula temperatura media dos compartimentos teciduais
*****************************************************************************/
void Segmento::CalculaTempCompTec()
{
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].ZeraTemperatura();
    }
    // Soma na temperatura T*V de cada celula
    for (IntPos i=0; i<lCelula.DimVet(); ++i) {
        lCompTecido[lCelula.Em(i)->NumCompCel()].SomaTempVol(
                lCelula.Em(i)->Volume()*lCelula.Em(i)->Temperatura());
    }
    // Divide por volume total do compartimento
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].DivideTempVolTotal();
    }
    // Atualiza temperatura do pulm�o
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            compPulmao.SetTemperatura(lCompTecido[i].Temperatura());
        }
    }

}

/*****************************************************************************
* Atualiza os coeficientes que variam ao longo da simula��o
*****************************************************************************/
void Segmento::AtualizaCoefVar()
{
    for (IntPos iC=0; iC<lCelula.DimVet(); ++iC) {
        //determina a temperatura do sangue na entrada da c�lula,
        //igual a temperatura do compartimento arterial
        //ou igual a temperatura do compartimento venoso no caso do pulm�o
        double tempEntrada = compArterial.Temperatura();
        if (lCelula.Em(iC)->NomeTecido()==PULMAO)
            tempEntrada = compVenoso.Temperatura();
        
        //atualiza os termos relativos a fonte das celulas,
        //inclui a vaz�o de sangue e a gera��o de calor
        lCelula.Em(iC)->SetFonteC(lCelula.Em(iC)->GerCalor() 
                +lCelula.Em(iC)->VazSangueEsp()*densidadeSangue
                *calorEspSangue*tempEntrada);
        lCelula.Em(iC)->SetFonteP(lCelula.Em(iC)->VazSangueEsp()
                *densidadeSangue*calorEspSangue*(-1.0));
    }
}

/*****************************************************************************
* Calcula temperatura no proximo instante do compartimento arterial
*****************************************************************************/
double Segmento::CalcTempArterial() 
{
    int numPulmao=0;
    bool temPulmao=false;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            temPulmao=true;
            numPulmao=i;
        }
    }
    //temperatura do sangue na entrada
    double tempEntrada = tempArtEnt;
    if (temPulmao) tempEntrada = lCompTecido[numPulmao].Temperatura();

    double numerador   = densidadeSangue*calorEspSangue*vazSangueAnt*tempEntrada
                         +coefArtVen*compVenoso.Temperatura();
    double denominador = densidadeSangue*calorEspSangue*vazSangueAnt 
                         +coefArtVen;
    
    if (!regimePermanente) {
        numerador   += compArterial.TemperaturaAnt()*compArterial.Volume()
                       *densidadeSangue*calorEspSangue/passoTempo;
        denominador += compArterial.Volume()*densidadeSangue
                       *calorEspSangue/passoTempo;
    }

    return numerador/denominador;
}

/*****************************************************************************
* Calcula temperatura no proximo instante do compartimento venoso
*****************************************************************************/
double Segmento::CalcTempVenoso() 
{
    double calorTecidos = 0.0;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()!=PULMAO) 
            calorTecidos += densidadeSangue*calorEspSangue
                *lCompTecido[i].VazSangue()
                *lCompTecido[i].Temperatura();
    }

    double numerador   = densidadeSangue*calorEspSangue*vazSanguePos
                         *tempVenEnt
                         +coefArtVen*compArterial.Temperatura()
                         +calorTecidos;
    double denominador = vazSangueAnt*densidadeSangue*calorEspSangue 
                         +coefArtVen;
    
    if (!regimePermanente) {
        numerador   += compVenoso.TemperaturaAnt()*compVenoso.Volume()
                       *densidadeSangue*calorEspSangue/passoTempo;
        denominador += compVenoso.Volume()*densidadeSangue
                       *calorEspSangue/passoTempo;
    }

    return numerador/denominador;
}

/*****************************************************************************
* Calcula press�o parcial dos gases nos compartimentos teciduais
*****************************************************************************/
Gas<double> Segmento::CalcPressaoTecido(IntPos iTec)
{
    Gas<double> novaPressao(0.0,0.0);

    if (regimePermanente) {
        Gas<double> novaConc;
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaConc(ig) = lCompTecido[iTec].GerGas(ig)
                           /lCompTecido[iTec].VazSangue()
                           + compArterial.Concentracao(ig);
        }
        novaConc.SetO2(novaConc.O2()*relaxaPres
                +lCompTecido[iTec].ConcPeqVasos(GO2)*(1.0-relaxaPres));
        novaConc.SetCO2(novaConc.CO2()*relaxaPres
                +lCompTecido[iTec].ConcPeqVasos(GCO2)*(1.0-relaxaPres));
        lCompTecido[iTec].SetConcPeqVasos(novaConc);
        lCompTecido[iTec].AtualizaPressao();
    }

    else /*if (!regimePermanente)*/ {

        Gas<double> derivadaSangue
            =lCompTecido[iTec].DerivadaConcentracaoSangue();
        Gas<double> derivadaTecido
            =lCompTecido[iTec].DerivadaConcentracaoTecido(
            lCompTecido[iTec].Pressao());

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {

            novaPressao.SetGas(ig,lCompTecido[iTec].PressaoAnt(ig)
                 + passoTempo
                 /(lCompTecido[iTec].VolPeqVasos()*derivadaSangue(ig)
                 + lCompTecido[iTec].VolTecido()*derivadaTecido(ig))
                 *(lCompTecido[iTec].VazSangue()
                 *(compArterial.Concentracao(ig)
                 - lCompTecido[iTec].ConcPeqVasos(ig))
                 + lCompTecido[iTec].GerGas(ig)));
        }
    }
    return novaPressao;

}

/*****************************************************************************
* Calcula press�o parcial dos gases no compartimento arterial
*****************************************************************************/
Gas<double> Segmento::CalcPressaoArterial()
{
    int numPulmao=0;
    bool temPulmao=false;
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            temPulmao=true;
            numPulmao=i;
            //cout << " pulmao ";
        }
    }
    //concentra��o do sangue na entrada
    Gas<double> concEntrada = concArtEnt;
    if (temPulmao) {
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            concEntrada(ig) = compPulmao.ConcUltimoCapPulm(ig)
                *compPulmao.Shunt()
                + compVenoso.Concentracao(ig)
                *(1.0-compPulmao.Shunt());
        }
    }

    Gas<double> novaPressao(0.0,0.0);

    if (regimePermanente) {
        compArterial.SetConcentracao(concEntrada);
        compArterial.AtualizaPressao();
    }
    else /*if (!regimePermanente)*/ {

        Gas<double> derivada = compArterial.DerivadaConcentracao();

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaPressao.SetGas(ig,compArterial.PressaoAnt(ig)
                 + passoTempo*vazSangueAnt
                 *(concEntrada(ig) - compArterial.Concentracao(ig))
                 /(compArterial.Volume()*derivada(ig)));
        }
    }
    return novaPressao;
}

/*****************************************************************************
* Calcula press�o parcial dos gases no compartimento venoso
*****************************************************************************/
Gas<double> Segmento::CalcPressaoVenoso()
{
    Gas<double> novaPressao(0.0,0.0);

    Gas<double> vazTec(0.0,0.0);
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()!=OSSO 
                && lCompTecido[i].NomeTecido()!=PULMAO) {
            for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                vazTec(ig) += lCompTecido[i].ConcPeqVasos(ig)
                    *lCompTecido[i].VazSangue();
            }
        }
    }

    if (regimePermanente) {
        Gas<double> novaConc;
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaConc(ig) = (vazTec(ig) + vazSanguePos*concVenEnt(ig))
                           /vazSangueAnt;
        }
        compVenoso.SetConcentracao(novaConc);
        compVenoso.AtualizaPressao();
    }

    else /*if (!regimePermanente)*/ {

        Gas<double> derivada = compVenoso.DerivadaConcentracao();

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaPressao.SetGas(ig,compVenoso.PressaoAnt(ig)
                 + passoTempo*(vazTec(ig) 
                 + vazSanguePos*concVenEnt(ig)
                 - vazSangueAnt*compVenoso.Concentracao(ig))
                 /(compVenoso.Volume()*derivada(ig)));
        }
    }
    return novaPressao;
}

/*****************************************************************************
* Calcula press�o parcial dos gases no compartimento alveolar
*****************************************************************************/
Gas<double> Segmento::CalcPressaoAlveolar()
{
    Gas<double> novaPressao(0.0,0.0);
    Gas<double> beta(0.0224,0.02226);
    
    Gas<double> somaPresCapPulm(0.0,0.0);
    for (IntPos k=0; k<compPulmao.NumCompCapPulm(); ++k) {
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            somaPresCapPulm(ig) += compPulmao.PresCapPulm(k,ig);
        }
    }
    //cout << " somaPcp: " << somaPresCapPulm.O2() << " " << somaPresCapPulm.CO2();

    if (regimePermanente) {
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaPressao(ig) = (ventilacao*presInspirada(ig)
                + beta(ig)*amb->PressBar()*compPulmao.CoefDifusao(ig)
                *somaPresCapPulm(ig))
                /(ventilacao 
                + beta(ig)*amb->PressBar()*compPulmao.CoefDifusaoTotal(ig));
        }
        //cout << " VA: " << ventilacao*60.0e6 << " pO2in: " << presInspirada(0)/133.32 << " ";
        //compPulmao.SetPressaoAlveolar(novaPressao);
    }

    else /*if (!regimePermanente)*/ {

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaPressao(ig) = (compPulmao.PresAlveolarAnt(ig)
                *compPulmao.VolAlveolar()/passoTempo
                + ventilacao*presInspirada(ig)
                + beta(ig)*amb->PressBar()*compPulmao.CoefDifusao(ig)
                *somaPresCapPulm(ig))
                /(compPulmao.VolAlveolar()/passoTempo
                + ventilacao + beta(ig)*amb->PressBar()
                *compPulmao.CoefDifusaoTotal(ig));
        }
    }
    return novaPressao;
}


/*****************************************************************************
* Calcula press�o parcial dos gases nos compartimentos capilares pulmonares
*****************************************************************************/
Gas<double> Segmento::CalcPressaoCapPulm(IntPos iCap, IntPos iTec)
{
    Gas<double> novaPressao(0.0,0.0);

    //se for o primeiro compartimento a entrada � de sangue venoso
    Gas<double>  concEntrada(0.0,0.0);
    if (iCap==0) concEntrada = compVenoso.Concentracao();
    else         concEntrada = compPulmao.ConcCapPulm(iCap-1);

    if (regimePermanente) {
        Gas<double> novaConc;
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            novaConc(ig) = (compPulmao.CoefDifusao(ig)
                           *(compPulmao.PresAlveolar(ig)
                           - compPulmao.PresCapPulm(iCap,ig))
                           + lCompTecido[iTec].GerGas(ig)
                           /compPulmao.NumCompCapPulm())
                           /(vazSangueAnt*compPulmao.Shunt())
                           + concEntrada(ig);
        }
        novaConc.SetO2(novaConc.O2()*relaxaPres
                + compPulmao.ConcCapPulm(iCap,GO2)*(1.0-relaxaPres));
        novaConc.SetCO2(novaConc.CO2()*relaxaPres
                + compPulmao.ConcCapPulm(iCap,GCO2)*(1.0-relaxaPres));
        compPulmao.SetConcCapPulm(iCap,novaConc);
        compPulmao.AtualizaPressao(iCap);
    }

    else /*if (!regimePermanente)*/ {

        Gas<double> derivadaSangue
            =compPulmao.DerivadaConcentracaoSangue(iCap);
        Gas<double> derivadaTecido(9.2928e-6,0.00056608);

        for (IntPos ig=0; ig<NUM_GAS; ++ig) {

            novaPressao(ig) = (compPulmao.PresCapPulmAnt(iCap,ig)
                + passoTempo
                *(vazSangueAnt*compPulmao.Shunt()
                *(concEntrada(ig) - compPulmao.ConcCapPulm(iCap,ig))
                + compPulmao.CoefDifusao(ig)*compPulmao.PresAlveolar(ig)
                + lCompTecido[iTec].GerGas(ig)/compPulmao.NumCompCapPulm())
                /(compPulmao.VolCapPulm()*derivadaSangue(ig)
                + compPulmao.VolTecido()*derivadaTecido(ig)))
                /(1.0 + passoTempo*compPulmao.CoefDifusao(ig)
                /(compPulmao.VolCapPulm()*derivadaSangue(ig)
                + compPulmao.VolTecido()*derivadaTecido(ig)));
        }
    }
    return novaPressao;
}

/*****************************************************************************
* Calcula temperatura das c�lulas
*****************************************************************************/
void Segmento::CalcTempCel()
{
	time_t tempoACV = time(NULL);
    AtualizaCoefVar();
    totTtempoACV += ( time(NULL) - tempoACV);

    time_t tempoCorpoCTC = time(NULL);


    calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
    double calAmb=0.0; //para uma celula
    double aP=0.0,
           aP0=0.0, 
           aW=0.0, 
           aE=0.0,
           aS=0.0,
           aN=0.0, 
           b=0.0; 
    double TW=0.0, 
           TE=0.0,
           TS=0.0,
           TN=0.0;
    bool ehFronteiraW=false, 
         ehFronteiraE=false;
    double dVolume=0.0;

    //usados para calcular temperatura superficial m�dia
    tempSuperficialMedia = 0.0;
    double areaTotal = 0.0;
   
    int i=0, j=0;
    bool varreduraCompleta = false;
    bool varreduraHorizontal = true;
    bool ida = true;

    if (geometria==RADIAL) {

        int i=0;
        bool ida = true;

        while (i>=0) {
            ehFronteiraW=false;
            ehFronteiraE=false;

            Celula *celula = lCelula.Em(i);
            Face *faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);

            double rw = faceW->Centro().C1();
            double re = faceE->Centro().C1();

            double novaTemp=0.0;

            dVolume = M_PI*(re*re-rw*rw);

            if (faceW->Tipo()==FACE_INT) {
                aW = faceW->CondTerm()*rw*2.0*M_PI/faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
            }
            else {
                aW = 0.0;
                ehFronteiraW=true;
            }
            
            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*re*2.0*M_PI/faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*re*2.0*M_PI;
                if (ida) {
                    double area = re*2.0*M_PI*celula->Tamanho().C3();
                    calorProAmbiente += calAmb*area;
                    areaTotal += area;
                    tempSuperficialMedia += faceE->Temperatura()*area;
                }
            }
            else if (ehFronteiraW) {
                aE *= 2.0;
            }
                
            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + b)/aP;
            
            //celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);

            if (i == int(lCelula.DimVet())-1) ida = false;
            if (ida) ++i;
            else --i;
        }
    }

    if (geometria==RETANGULAR) {
/*
    	boost::mutex m;
    	boost::barrier b(3);

    	int k = lCelula.Dim(0)/2;

        calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
        tempSuperficialMedia = 0.0;

        calcTempCelRet(0, (int)lCelula.Dim(0), 0, (int)lCelula.Dim(0), m, b);
*/


        bool ehFronteiraS=false, ehFronteiraN=false;

        while (!varreduraCompleta) {

            ehFronteiraW=false;
            ehFronteiraE=false;
            ehFronteiraS=false;
            ehFronteiraN=false;

            Celula *celula = lCelula.Em(i,j);
            Face *faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);
            Face *faceS = celula->FaceSentido(S);
            Face *faceN = celula->FaceSentido(N);

            double novaTemp = 0.0;

            dVolume = celula->Tamanho().C1()*celula->Tamanho().C2();

            if (faceW->Tipo()==FACE_INT) {
                aW = faceW->CondTerm()*celula->Tamanho().C2()
                    /faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
            }
            else {
                aW = 0.0;
                ehFronteiraW=true;
            }

            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*celula->Tamanho().C2()
                    /faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            if (faceS->Tipo()==FACE_INT) {
                aS = faceS->CondTerm()*celula->Tamanho().C1()
                    /faceS->DCelula();
                TS = celula->CelVizSentido(S)->Temperatura();
            }
            else {
                aS = 0.0;
                ehFronteiraS=true;
            }

            if (faceN->Tipo()==FACE_INT) {
                aN = faceN->CondTerm()*celula->Tamanho().C1()
                    /faceN->DCelula();
                TN = celula->CelVizSentido(N)->Temperatura();
            }
            else {
                aN = 0.0;
                ehFronteiraN=true;
            }

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    calorProAmbiente += calAmb*area;
                    areaTotal += area;
                    tempSuperficialMedia += faceE->Temperatura()*area;
                }
            }
            else if (ehFronteiraW) {
                calAmb = CalorAmbiente(faceW->Temperatura());
                faceW->SetTemperatura(calAmb*faceW->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    calorProAmbiente += calAmb*area;
                    areaTotal += area;
                    tempSuperficialMedia += faceW->Temperatura()*area;
                }
            }

            if (ehFronteiraS) {
                calAmb = CalorAmbiente(faceS->Temperatura());
                faceS->SetTemperatura(calAmb*faceS->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) { //deveria ser na volta
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    calorProAmbiente += calAmb*area;
                    areaTotal += area;
                    tempSuperficialMedia += faceS->Temperatura()*area;
                }
            }
            else if (ehFronteiraN) {
                calAmb = CalorAmbiente(faceN->Temperatura());
                faceN->SetTemperatura(calAmb*faceN->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    calorProAmbiente += calAmb*area;
                    areaTotal += area;
                  tempSuperficialMedia += faceN->Temperatura()*area;
                }
            }

            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aS + aN + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + aS*TS + aN*TN + b)/aP;

            celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);

            if (varreduraHorizontal) {
                ++i;
                if (i == int(lCelula.Dim(0))) {
                    i=0;
                    ++j;
                }
                if (j==int(lCelula.Dim(1))) {
                    varreduraHorizontal = false;
                    j=0;
                    i=0;
                }
            }
            else {
                ++j;
                if (j == int(lCelula.Dim(1))) {
                    j=0;
                    ++i;
                }
                if (i==int(lCelula.Dim(0))) varreduraCompleta = true;
            }
        }



//    	testectcr();


    }

    if (geometria==POLAR) {
        
        i = int(lCelula.Dim(0))-1;
        while (!varreduraCompleta) {
            bool ehCentro=false;
            ehFronteiraE=false;
            
            Celula *celula = lCelula.Em(i,j);
            if (celula->EhCentro()) ehCentro=true; 
            Face *faceW = 0;
            if (!ehCentro) faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);
            Face *faceS = celula->FaceSentido(S);
            Face *faceN = celula->FaceSentido(N);

            double novaTemp=0.0;

            double rw = 0.0;
            if (!ehCentro) rw = faceW->Centro().C1();
            double re = faceE->Centro().C1();
            double rs = faceS->Centro().C1();
            double rn = faceN->Centro().C1();
            
            dVolume = celula->Tamanho().C2()*(re*re-rw*rw)/2.0;

            if (!ehCentro) {
                aW = faceW->CondTerm()*rw*celula->Tamanho().C2()
                    /faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
                }
            else aW = 0.0;
            
            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*re*celula->Tamanho().C2()
                    /faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            aS = faceS->CondTerm()*celula->Tamanho().C1()
                /(faceS->DCelula()*rs);
            TS = celula->CelVizSentido(S)->Temperatura();
            
            aN = faceN->CondTerm()*celula->Tamanho().C1()
                /(faceN->DCelula()*rn);
            TN = celula->CelVizSentido(N)->Temperatura();

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*re*celula->Tamanho().C2();
                    
                double area = re*celula->Tamanho().C2()*celula->Tamanho().C3();
                calorProAmbiente += calAmb*area;
                areaTotal += area;
                tempSuperficialMedia += faceE->Temperatura()*area;
            }

            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aS + aN + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + aS*TS + aN*TN + b)/aP;

            //celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);

            if (i == int(lCelula.Dim(0))-1) ida = false;
            if (ida) ++i;
            else --i;
            if (i<0) {
                ++j;
                i=0;
                ida=true;
            }
            if (j==int(lCelula.Dim(1))) varreduraCompleta = true;

        }
    } //Polar


    if (geometria==AXIAL) {

        bool ehFronteiraS=false, 
             ehFronteiraN=false;

        while (!varreduraCompleta) {
            ehFronteiraW=false;
            ehFronteiraE=false;
            ehFronteiraS=false;
            ehFronteiraN=false;

            Celula *celula = lCelula.Em(i,j);
            Face *faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);
            Face *faceS = celula->FaceSentido(S);
            Face *faceN = celula->FaceSentido(N);

            double rw = faceW->Centro().C1();
            double re = faceE->Centro().C1();

            double novaTemp=0.0;

            dVolume = (re*re-rw*rw)*M_PI*celula->Tamanho().C2();

            if (faceW->Tipo()==FACE_INT) {
                aW = faceW->CondTerm()*rw*2.0*M_PI*celula->Tamanho().C2()
                    /faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
            }
            else {
                aW = 0.0;
                ehFronteiraW=true;
            }
            
            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*re*2.0*M_PI*celula->Tamanho().C2()
                    /faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            if (faceS->Tipo()==FACE_INT) {
                aS = faceS->CondTerm()*(re*re-rw*rw)*M_PI/faceS->DCelula();
                TS = celula->CelVizSentido(S)->Temperatura();
            }
            else {
                aS = 0.0;
                ehFronteiraS=true;
            }

            if (faceN->Tipo()==FACE_INT) {
                aN = faceN->CondTerm()*(re*re-rw*rw)*M_PI/faceN->DCelula();
                TN = celula->CelVizSentido(N)->Temperatura();
            }
            else {
                aN = 0.0;
                ehFronteiraN=true;
            }

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*re*celula->Tamanho().C2()*2.0*M_PI;

                double area = re*2.0*M_PI*celula->Tamanho().C2();
                calorProAmbiente += calAmb*area;
                areaTotal += area;
                tempSuperficialMedia += faceE->Temperatura()*area;
            }
            else if (ehFronteiraW) aE *= 2.0;

            if (ehFronteiraS) aN *= 2.0;
            else if (ehFronteiraN) aS *= 2.0;

            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aS + aN + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + aS*TS + aN*TN + b)/aP;

            //celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);
            
            ++i;
            if (i == int(lCelula.Dim(0))) {
                i=0;
                ++j;
            }
            if (j==int(lCelula.Dim(1))) varreduraCompleta = true;

        }
    }

    totTempoCorpoCTC += ( time(NULL) - tempoCorpoCTC );


    time_t tempoCTCT = time(NULL);
    CalculaTempCompTec();
    totTempoCTCT += (time(NULL) - tempoCTCT);

    tempSuperficialMedia /= areaTotal;

}

/*****************************************************************************
* Da um passo em tudo
*****************************************************************************/
void Segmento::PassoCalculo()
{
	PassoCalculoTemperatura();

	AtualizaEntreCalculos();

	PassoCalculoPressao();
}

void Segmento::AtualizaEntreCalculos()
{
	for (IntPos i=0; i<lCompTecido.size(); ++i) {
		if (lCompTecido[i].NomeTecido()==PULMAO) {
			lCompTecido[i].SetGerCalorEsp(339.0);
		}
		else if (lCompTecido[i].NomeTecido()==MUSCULO) {
				lCompTecido[i].SetGerCalorEsp(geracaoCalorMusculo);
		}
	}
}

/*****************************************************************************
* Da um passo no calculo da temperatura
*****************************************************************************/
void Segmento::PassoCalculoTemperatura()
{
    double atingiuPrecisao = true;
    double erro = 0.0;
    double tempAux = 0.0;

    // Calculo da temperatura do compartimento arterial

    time_t tempoCTA = time(NULL);
    tempAux = CalcTempArterial();
    totTempoCTA += ( time(NULL) -  tempoCTA);

    erro = fabs((tempAux-compArterial.Temperatura())/tempAux);
    if (erro > PRECISAO) atingiuPrecisao = false;
    compArterial.SetTemperatura(tempAux);

    // Calculo da temperatura do compartimento venoso

    time_t tempoCTV = time(NULL);
    tempAux = CalcTempVenoso();
    totTempoCTV += ( time(NULL) -  tempoCTV);

    erro = fabs((tempAux-compVenoso.Temperatura())/tempAux);
    if (erro > PRECISAO) atingiuPrecisao = false;
    compVenoso.SetTemperatura(tempAux);
    
    // Calculo da temperatura das c�lulas

    time_t tempoAM = time(NULL);
    Matriz<double> matTempAux(lCelula.Dim(0),lCelula.Dim(1));
    lCelula.AtualizaMatrizTemperatura(matTempAux);
    totTempoAM += ( time(NULL) -  tempoAM);

    time_t tempoCTC = time(NULL);
    CalcTempCelSwt();
    totTempoCTC += ( time(NULL) -  tempoCTC);

    for (IntPos i=0; i<lCelula.DimVet(); ++i) {
        erro = fabs((matTempAux(i)-lCelula.Em(i)->Temperatura())
                    /matTempAux(i));
        if (erro > PRECISAO) atingiuPrecisao = false;
    }

    setAtingiuPrecisaoCalcTemp(atingiuPrecisao);
    //return atingiuPrecisao;
}


/*****************************************************************************
* Da um passo no calculo das press�es parciais
*****************************************************************************/
void Segmento::PassoCalculoPressao()
{
    double atingiuPrecisao = true;
    double erro = 0.0;
    Gas<double> presAux;

    //cout << " -" << lCompTecido.size() << "- ";

    //cout << " Pve:" 
    //     << compVenoso.Pressao(0)/133.32 << "/" 
    //     << compVenoso.Pressao(1)/133.32;

    // Calculo da press�o parcial do compartimento arterial

    presAux = CalcPressaoArterial();

    if (!regimePermanente) { //acrescentar precisao para RP, com concentracao
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            erro = fabs((presAux(ig)-compArterial.Pressao(ig))/presAux(ig));
            if (erro > PRECISAO) atingiuPrecisao = false;
        }
        presAux.SetO2(presAux.O2()*relaxaPres
                +compArterial.Pressao(GO2)*(1.0-relaxaPres));
        presAux.SetCO2(presAux.CO2()*relaxaPres
                +compArterial.Pressao(GCO2)*(1.0-relaxaPres));

        compArterial.SetPressao(presAux);
        compArterial.AtualizaConcentracao();
    }

    //cout << " Par:" 
    //     << compArterial.Pressao(0)/133.32 << "/" 
    //     << compArterial.Pressao(1)/133.32;


    // Calculo da press�o parcial dos tecidos
    //cout << " Pt:";

    boost::thread * threadPulmao = NULL;
    bool temPulmao = false;

    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        //Se for pulmao
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            //Pressao alveolar

        	time_t tempoPulmao = time(NULL);

        	temPulmao = true;

        	presAux = CalcPressaoAlveolar();


            for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                erro = fabs((presAux(ig)-compPulmao.PresAlveolar(ig))
                        /presAux(ig));
                if (erro > PRECISAO) atingiuPrecisao = false;
            }
            presAux.SetO2(presAux.O2()*relaxaPres
                          +compPulmao.PresAlveolar(GO2)*(1.0-relaxaPres));
            presAux.SetCO2(presAux.CO2()*relaxaPres
                           +compPulmao.PresAlveolar(GCO2)*(1.0-relaxaPres));
            compPulmao.SetPresAlveolar(presAux);
            //cout << " pAux: " << presAux.O2() << " " << presAux.CO2() << " ";
            //Capilares pulmonares

            threadPulmao = new boost::thread(&Segmento::AtualizaCapilaresPulmonar, this, i);
            //threadPulmao->join();
            //AtualizaCapilaresPulmonar(i);


        } //se pulm�o
        
        else if (lCompTecido[i].NomeTecido()!=OSSO) {

            presAux = CalcPressaoTecido(i);

            if (!regimePermanente) {
                for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                    erro = fabs((presAux(ig)-lCompTecido[i].Pressao(ig))
                            /presAux(ig));
                    if (erro > PRECISAO) atingiuPrecisao = false;
                }
                presAux.SetO2(presAux.O2()*relaxaPres
                        +lCompTecido[i].Pressao(GO2)*(1.0-relaxaPres));
                presAux.SetCO2(presAux.CO2()*relaxaPres
                        +lCompTecido[i].Pressao(GCO2)*(1.0-relaxaPres));
                lCompTecido[i].SetPressao(presAux);
                lCompTecido[i].AtualizaConcentracao();
            } //se RT
        } //se outros tecidos

        //cout << lCompTecido[i].Pressao(0)/133.32 << "/" 
        //     << lCompTecido[i].Pressao(1)/133.32 << " ";

    } //for compartimentos teciduais


    if ( temPulmao )
       	threadPulmao->join();


    // Calculo da press�o parcial do compartimento venoso
    presAux = CalcPressaoVenoso();

    if (!regimePermanente) {
        for (IntPos ig=0; ig<NUM_GAS; ++ig) {
            erro = fabs((presAux(ig)-compVenoso.Pressao(ig))/presAux(ig));
            if (erro > PRECISAO) atingiuPrecisao = false;
        }
        presAux.SetO2(presAux.O2()*relaxaPres
                +compVenoso.Pressao(GO2)*(1.0-relaxaPres));
        presAux.SetCO2(presAux.CO2()*relaxaPres
                +compVenoso.Pressao(GCO2)*(1.0-relaxaPres));

        compVenoso.SetPressao(presAux);
        compVenoso.AtualizaConcentracao();
    }

    //cout << " Pve:" 
    //     << compVenoso.Pressao(0)/133.32 << "/" 
    //     << compVenoso.Pressao(1)/133.32;

    //cout << " PA:" 
    //     << compPulmao.PresAlveolar(0)/133.32 << "/" 
    //     << compPulmao.PresAlveolar(1)/133.32;

    //cout << endl;

    //return atingiuPrecisao;
    //return true;

    //if ( temPulmao )
//    	threadPulmao->join();

    setAtingiuPrecisaoCalcPressao(true);
}

void Segmento::AtualizaCapilaresPulmonar(int i)
{
	time_t tempoForPulmao = time(NULL);

	if ( regimePermanente )
	{
		//rpc++;
		AtualizaCapilaresPulmonarSeq(i);
	}
	else
	{
		//nrpc++;
		boost::thread_group threadsCaps;

	    int const numGrupoSeg = 4;
	    int const numeroDeCapilares = compPulmao.NumCompCapPulm();

	    vector<int> grupo1;
	    vector<int> grupo2;
	    vector<int> grupo3;
	    vector<int> grupo4;

	    vector< vector<int> > grupos;
	    grupos.push_back(grupo1);
	    grupos.push_back(grupo2);
	    grupos.push_back(grupo3);
	    grupos.push_back(grupo4);

	    int indCap = 0;

	    for (int l = 0; l < 4; l++)
	    {
	    	for (int j = 0; j < numeroDeCapilares / numGrupoSeg; ++j )
	    	{
	    		grupos[l].push_back(indCap);
	    		++indCap;
	    	}
	    }
	    for (int l = indCap; l < numeroDeCapilares; ++l) grupo4.push_back(l);

		for (IntPos k = 0; k < numGrupoSeg; ++k) {
			threadsCaps.add_thread(new boost::thread(&Segmento::AtualizaCapilaresPulmonarPar, this, boost::ref(grupos[k]), i));
		}
		threadsCaps.join_all();

	     time_t tempoAtConc = time(NULL);

		compPulmao.AtualizaConcentracao();

	     tempoTotAtConc += (time(NULL) - tempoAtConc);

	}

	tempoTotForPulmao += ( time(NULL) - tempoForPulmao );

	/*
	boost::thread_group threadsCaps;
	for (IntPos k=0; k<compPulmao.NumCompCapPulm(); ++k) {
		threadsCaps.add_thread(new boost::thread(&Segmento::teste, this, k, i));
	}
	threadsCaps.join_all();

	compPulmao.AtualizaConcentracao();

	*/

//	AtualizaCapilaresPulmonarSeq(i);


	/*
    time_t tempoForPulmao = time(NULL);

     totExecCompPulm++;

     Gas<double> presAux;

     for (IntPos k=0; k<compPulmao.NumCompCapPulm(); ++k) {
         presAux = CalcPressaoCapPulm(k,i);
         if (!regimePermanente) {
             presAux.SetO2(presAux.O2()*relaxaPres
                     +compPulmao.PresCapPulm(k,GO2)*(1.0-relaxaPres));
             presAux.SetCO2(presAux.CO2()*relaxaPres
                     +compPulmao.PresCapPulm(k,GCO2)*(1.0-relaxaPres));
             compPulmao.SetPresCapPulm(k,presAux);


             for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                 erro = fabs((presAux(ig)-compPulmao.PresCapPulm(k,ig))
                         /presAux(ig));
                 if (erro > PRECISAO) atingiuPrecisao = false;
             }
             *-/


         } //se RT
     } //for capilar pulmonar

     tempoTotForPulmao += ( time(NULL) - tempoForPulmao );

     time_t tempoAtConc = time(NULL);

     compPulmao.AtualizaConcentracao();

     tempoTotAtConc += (time(NULL) - tempoAtConc);

     //tempoTotPulmao += ( time(NULL) - tempoPulmao);
      * */


}


void Segmento::AtualizaCapilaresPulmonarSeq(int i)
{



     totExecCompPulm++;

     Gas<double> presAux;

     for (IntPos k=0; k<compPulmao.NumCompCapPulm(); ++k) {
         presAux = CalcPressaoCapPulm(k,i);
         if (!regimePermanente) {
             presAux.SetO2(presAux.O2()*relaxaPres
                     +compPulmao.PresCapPulm(k,GO2)*(1.0-relaxaPres));
             presAux.SetCO2(presAux.CO2()*relaxaPres
                     +compPulmao.PresCapPulm(k,GCO2)*(1.0-relaxaPres));
             compPulmao.SetPresCapPulm(k,presAux);

             /*
             for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                 erro = fabs((presAux(ig)-compPulmao.PresCapPulm(k,ig))
                         /presAux(ig));
                 if (erro > PRECISAO) atingiuPrecisao = false;
             }
             */


         } //se RT
     } //for capilar pulmonar



     time_t tempoAtConc = time(NULL);

     compPulmao.AtualizaConcentracao();

     tempoTotAtConc += (time(NULL) - tempoAtConc);

     //tempoTotPulmao += ( time(NULL) - tempoPulmao);

}



void Segmento::AtualizaCapilaresPulmonarPar(vector<int> indiceCapilares, int i)
{

	int numeroDeCapilares = indiceCapilares.size();

	for (int j = 0; j < numeroDeCapilares; ++j) {

		int k = indiceCapilares[j];

		Gas<double> presAux;

		presAux = CalcPressaoCapPulm(k,i);
		if (!regimePermanente) {
			presAux.SetO2(presAux.O2()*relaxaPres
					+compPulmao.PresCapPulm(k,GO2)*(1.0-relaxaPres));
			presAux.SetCO2(presAux.CO2()*relaxaPres
					+compPulmao.PresCapPulm(k,GCO2)*(1.0-relaxaPres));
			compPulmao.SetPresCapPulm(k,presAux);

			/*
             for (IntPos ig=0; ig<NUM_GAS; ++ig) {
                 erro = fabs((presAux(ig)-compPulmao.PresCapPulm(k,ig))
                         /presAux(ig));
                 if (erro > PRECISAO) atingiuPrecisao = false;
             }
			 */


		} //se RT

	}
}


/*****************************************************************************
* Atualiza valores anteriores com atuais
*****************************************************************************/
void Segmento::AtualizaValorAnt()
{
    compArterial.AtualizaTemperaturaAnt();
    compArterial.AtualizaPressaoAnt();
    compVenoso.AtualizaTemperaturaAnt();
    compVenoso.AtualizaPressaoAnt();
    lCelula.AtualizaTempAntiga();
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            compPulmao.AtualizaPressaoAnt();
        }
        else lCompTecido[i].AtualizaPressaoAnt();
    }
}

/*****************************************************************************
* Calor trocado na respira��o
*****************************************************************************/
double Segmento::CalorRespiracao(double tempAr, double uRel)
{
    double umidadeAbs = amb->UmidadeAbs(tempAr,uRel,amb->PressBar());
    double tempExp = 32.6 + 0.066*tempAr + 32.0*umidadeAbs;
    double umidExp = 0.0277 + 0.000065*tempAr + 0.2*umidadeAbs;
    return ventilacao*amb->DensidadeAr(tempAr,amb->PressBar())*
        (amb->EntalpiaVapAgua(tempAr)*(umidExp-umidadeAbs) 
        + amb->CalorEspAr(tempAr)*(tempExp-tempAr));
}

/*****************************************************************************
* Calor trocado com o ambiente
*****************************************************************************/
double Segmento::CalorAmbiente(double tSuperficial)
{
    return resistTemperatura*(amb->TempOperativa()-tSuperficial)
           +supMolhada*resistEvaporacao*
           (amb->UmidadeRel()*amb->PressVaporAgua(amb->TempOperativa())
           -amb->PressVaporAgua(tSuperficial));
}

/*****************************************************************************
* 
*****************************************************************************/
double Segmento::AreaSupCelulas()
{
    double area=0.0;
    for (IntPos iC=0; iC<lCelula.DimVet(); ++iC) {
        for (IntPos iF=0; iF<lCelula.Em(iC)->NVizinho(); ++iF) {
            if (lCelula.Em(iC)->FaceVizinho(iF)->Tipo()==FACE_EXT) {
                area+=lCelula.Em(iC)->FaceVizinho(iF)->AreaSuperficial(sistCoord);
            }
        }
    }
    return area;
}

/*****************************************************************************
* 
*****************************************************************************/
void Segmento::AjustaSupMolhada(double tPeleMedia, double tInterna)
{
    double calorMax = resistEvaporacao*
                      (amb->PressVaporAgua(tempSuperficialMedia)
                      -amb->UmidadeRel()
                      *amb->PressVaporAgua(amb->TempOperativa()));

    double tPeleLocal = TempCompTecido(PELE);

    supMolhada = 0.06 
        + 0.94*Sudorese(tPeleMedia,tPeleLocal,tInterna)/calorMax;	
    if (supMolhada < 0.06) supMolhada = 0.06;
    if (supMolhada > 1.0) supMolhada = 1.0;
}

/*****************************************************************************
* 
*****************************************************************************/
double Segmento::Sudorese(double tempPeleMedia, 
                          double tempPeleLocal, 
                          double tempInterna)
{
    double dTempPele = tempPeleMedia - 34.1;
    double dTempPeleLocal = tempPeleLocal - 34.1;
    double dTempInterna = tempInterna - 36.6;

    if (dTempPele < 0.0) dTempPele = 0.0;
    if (dTempPeleLocal < 0.0) dTempPeleLocal = 0.0;
    if (dTempInterna < 0.0) dTempInterna = 0.0;
    
    double cte1 = 100.0,
           cte2 = 11.0;

    return (cte1*dTempInterna + cte2*dTempPele)*exp(dTempPeleLocal/10.0);

}

/****************************************************************************************
 * Funcoes de concorrencia
 */

void Segmento::calcTempCelRet(int iniI, int limiteI, int iniJ, int limiteJ, boost::mutex &m, boost::barrier &b) {


}

/*****************************************************************************
*
*****************************************************************************/
//double Segmento::Amortiza(double valorAntigo, double valorAtual, double valorNovo)


// Funcao amortizar resultado
        //algoritmo para amortizar resultado
        //usar para concentra��es
        /*if (fabs(tempNova(i)-tempAtual(i))>fabs(tempAtual(i)-tempAntiga(i))) {
            //cout << "+";
            double valorPonderado=0.0;
            for (double fator=0.5; fator>0.01; fator/=2.0) {
                valorPonderado = tempNova(i)*fator + tempAtual(i)*(1.0-fator);
                if (fabs(valorPonderado-tempAtual(i))
                        <fabs(tempAtual(i)-tempAntiga(i))) break;
                lCelula.Em(i)->SetTemperatura(valorPonderado);
            }
        }*/
    //tempAntiga = tempAtual;
// -------------------------------------------------------------------------


/*****************************************************************************
* 
*****************************************************************************/
void Segmento::SaidaDadosArquivo(ostream &arquivo)
{
    arquivo << vazSangueAnt*60.0e6 << TAB;
    arquivo << vazSanguePos*60.0e6 << TAB;
    arquivo << vazSanguePeqVas*60.0e6 << TAB;
    arquivo << supMolhada << TAB;
    arquivo << tempSuperficialMedia << TAB;
    arquivo << calorProAmbiente << TAB;
    arquivo << concArtEnt.O2() << TAB;
    arquivo << concArtEnt.CO2() << TAB;
    arquivo << concVenEnt.O2() << TAB;
    arquivo << concVenEnt.CO2() << TAB;
    compArterial.SaidaDadosArquivo(arquivo);
    compVenoso.SaidaDadosArquivo(arquivo);
    for (IntPos i=0; i<lCompTecido.size(); ++i) {
        lCompTecido[i].SaidaDadosArquivo(arquivo);
        if (lCompTecido[i].NomeTecido()==PULMAO) {
            compPulmao.SaidaDadosArquivo(arquivo);
        }
    }

}

void Segmento::CalcTempCelAf(int iniI, int iniJ, int limI, int limJ, boost::mutex& m, boost::barrier& bHor, boost::barrier& bVert, boost::barrier& bProg)
{


/*
    	boost::mutex m;
    	boost::barrier b(3);

    	int k = lCelula.Dim(0)/2;

        calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
        tempSuperficialMedia = 0.0;

        calcTempCelRet(0, (int)lCelula.Dim(0), 0, (int)lCelula.Dim(0), m, b);
*/




    double localAreaTotal = 0.0;
    double  localCalorProAmbiente = 0.0, localTempSuperficialMedia = 0.0;

    for(int cont = 0; cont < 1; ++cont) {


        double calAmb=0.0; //para uma celula
        double aP=0.0,
               aP0=0.0,
               aW=0.0,
               aE=0.0,
               aS=0.0,
               aN=0.0,
               b=0.0;
        double TW=0.0,
               TE=0.0,
               TS=0.0,
               TN=0.0;
        bool ehFronteiraW=false,
             ehFronteiraE=false;
        double dVolume=0.0;

        //usados para calcular temperatura superficial m�dia



        int i=iniI, j=iniJ;
        bool varreduraCompleta = false;
        bool varreduraHorizontal = true;
        bool ida = true;



/*
    	boost::mutex m;
    	boost::barrier b(3);

    	int k = lCelula.Dim(0)/2;

        calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
        tempSuperficialMedia = 0.0;

        calcTempCelRet(0, (int)lCelula.Dim(0), 0, (int)lCelula.Dim(0), m, b);
*/


        bool ehFronteiraS=false, ehFronteiraN=false;

        while (!varreduraCompleta) {

            ehFronteiraW=false;
            ehFronteiraE=false;
            ehFronteiraS=false;
            ehFronteiraN=false;

            Celula *celula = lCelula.Em(i,j);
            Face *faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);
            Face *faceS = celula->FaceSentido(S);
            Face *faceN = celula->FaceSentido(N);

            double novaTemp = 0.0;

            dVolume = celula->Tamanho().C1()*celula->Tamanho().C2();

            if (faceW->Tipo()==FACE_INT) {
                aW = faceW->CondTerm()*celula->Tamanho().C2()
                    /faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
            }
            else {
                aW = 0.0;
                ehFronteiraW=true;
            }

            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*celula->Tamanho().C2()
                    /faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            if (faceS->Tipo()==FACE_INT) {
                aS = faceS->CondTerm()*celula->Tamanho().C1()
                    /faceS->DCelula();
                TS = celula->CelVizSentido(S)->Temperatura();
            }
            else {
                aS = 0.0;
                ehFronteiraS=true;
            }

            if (faceN->Tipo()==FACE_INT) {
                aN = faceN->CondTerm()*celula->Tamanho().C1()
                    /faceN->DCelula();
                TN = celula->CelVizSentido(N)->Temperatura();
            }
            else {
                aN = 0.0;
                ehFronteiraN=true;
            }

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceE->Temperatura()*area;
                }
            }
            else if (ehFronteiraW) {
                calAmb = CalorAmbiente(faceW->Temperatura());
                faceW->SetTemperatura(calAmb*faceW->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceW->Temperatura()*area;
                }
            }

            if (ehFronteiraS) {
                calAmb = CalorAmbiente(faceS->Temperatura());
                faceS->SetTemperatura(calAmb*faceS->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) { //deveria ser na volta
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceS->Temperatura()*area;
                }
            }
            else if (ehFronteiraN) {
                calAmb = CalorAmbiente(faceN->Temperatura());
                faceN->SetTemperatura(calAmb*faceN->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                  localTempSuperficialMedia += faceN->Temperatura()*area;
                }
            }

            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aS + aN + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + aS*TS + aN*TN + b)/aP;

            celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);

            if (varreduraHorizontal) {
                ++i;
                if (i == limI){//int(lCelula.Dim(0))) {
                    i=iniI;
                    ++j;
                }
                if (j==limJ){//int(lCelula.Dim(1))) {
                    varreduraHorizontal = false;
                    //j=0 ;
                    //i=0;
                    i = iniJ;
                    j = iniI;
                    bHor.wait();
                    bProg.wait();
                }
            }
            else {
                ++j;
                if (j == limI){//int(lCelula.Dim(1))) {
                    //j=0;
                	j = iniI;
                    ++i;
                }
                if (i==limJ)//int(lCelula.Dim(0)))
                {
                	varreduraCompleta = true;
					bVert.wait();
                }
            }
        }
    }
    {
    	boost::mutex::scoped_lock lock(m);
    	calorProAmbiente += localCalorProAmbiente;
    	tempSuperficialMedia += localTempSuperficialMedia;
    	areaTotal += localAreaTotal;
    }


//    	testectcr();



    //totTempoCorpoCTC += ( time(NULL) - tempoCorpoCTC );







}

void Segmento::CalcTempCelAfMid(int iniI, int iniJ, int limI, int limJ, boost::mutex& m, boost::barrier& bHor, boost::barrier& bVert, boost::barrier& bProg)
{


/*
    	boost::mutex m;
    	boost::barrier b(3);

    	int k = lCelula.Dim(0)/2;

        calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
        tempSuperficialMedia = 0.0;

        calcTempCelRet(0, (int)lCelula.Dim(0), 0, (int)lCelula.Dim(0), m, b);
*/
    double localAreaTotal = 0.0;
    double  localCalorProAmbiente = 0.0, localTempSuperficialMedia = 0.0;


    for(int cont = 0; cont < 1; ++cont) {



        double calAmb=0.0; //para uma celula
        double aP=0.0,
               aP0=0.0,
               aW=0.0,
               aE=0.0,
               aS=0.0,
               aN=0.0,
               b=0.0;
        double TW=0.0,
               TE=0.0,
               TS=0.0,
               TN=0.0;
        bool ehFronteiraW=false,
             ehFronteiraE=false;
        double dVolume=0.0;

        //usados para calcular temperatura superficial m�dia



        int i=iniI, j=iniJ;
        bool varreduraCompleta = false;
        bool varreduraHorizontal = true;
        bool ida = true;






/*
    	boost::mutex m;
    	boost::barrier b(3);

    	int k = lCelula.Dim(0)/2;

        calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
        tempSuperficialMedia = 0.0;

        calcTempCelRet(0, (int)lCelula.Dim(0), 0, (int)lCelula.Dim(0), m, b);
*/


        bool ehFronteiraS=false, ehFronteiraN=false;

        bHor.wait();

        while (!varreduraCompleta) {

            ehFronteiraW=false;
            ehFronteiraE=false;
            ehFronteiraS=false;
            ehFronteiraN=false;

            Celula *celula = lCelula.Em(i,j);
            Face *faceW = celula->FaceSentido(W);
            Face *faceE = celula->FaceSentido(E);
            Face *faceS = celula->FaceSentido(S);
            Face *faceN = celula->FaceSentido(N);

            double novaTemp = 0.0;

            dVolume = celula->Tamanho().C1()*celula->Tamanho().C2();

            if (faceW->Tipo()==FACE_INT) {
                aW = faceW->CondTerm()*celula->Tamanho().C2()
                    /faceW->DCelula();
                TW = celula->CelVizSentido(W)->Temperatura();
            }
            else {
                aW = 0.0;
                ehFronteiraW=true;
            }

            if (faceE->Tipo()==FACE_INT) {
                aE = faceE->CondTerm()*celula->Tamanho().C2()
                    /faceE->DCelula();
                TE = celula->CelVizSentido(E)->Temperatura();
            }
            else {
                aE = 0.0;
                ehFronteiraE=true;
            }

            if (faceS->Tipo()==FACE_INT) {
                aS = faceS->CondTerm()*celula->Tamanho().C1()
                    /faceS->DCelula();
                TS = celula->CelVizSentido(S)->Temperatura();
            }
            else {
                aS = 0.0;
                ehFronteiraS=true;
            }

            if (faceN->Tipo()==FACE_INT) {
                aN = faceN->CondTerm()*celula->Tamanho().C1()
                    /faceN->DCelula();
                TN = celula->CelVizSentido(N)->Temperatura();
            }
            else {
                aN = 0.0;
                ehFronteiraN=true;
            }

            b = celula->FonteC()*dVolume;

            if (ehFronteiraE) {
                calAmb = CalorAmbiente(faceE->Temperatura());
                faceE->SetTemperatura(calAmb*faceE->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceE->Temperatura()*area;
                }
            }
            else if (ehFronteiraW) {
                calAmb = CalorAmbiente(faceW->Temperatura());
                faceW->SetTemperatura(calAmb*faceW->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C2();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C2()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceW->Temperatura()*area;
                }
            }

            if (ehFronteiraS) {
                calAmb = CalorAmbiente(faceS->Temperatura());
                faceS->SetTemperatura(calAmb*faceS->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) { //deveria ser na volta
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                    localTempSuperficialMedia += faceS->Temperatura()*area;
                }
            }
            else if (ehFronteiraN) {
                calAmb = CalorAmbiente(faceN->Temperatura());
                faceN->SetTemperatura(calAmb*faceN->DCelula()
                        /celula->CondTerm() + celula->Temperatura());
                b += calAmb*celula->Tamanho().C1();
                if (!varreduraHorizontal && ida) {
                    double area =celula->Tamanho().C1()*celula->Tamanho().C3();
                    localCalorProAmbiente += calAmb*area;
                    localAreaTotal += area;
                  localTempSuperficialMedia += faceN->Temperatura()*area;
                }
            }

            if (regimePermanente) aP0 = 0.0;
            else /*if (!regimePermanente)*/ {
                aP0 = celula->Densidade()*celula->CalorEsp()*dVolume
                      /passoTempo;
                b += aP0*celula->TemperaturaAnt();
            }

            aP = aW + aE + aS + aN + aP0 - celula->FonteP()*dVolume;
            novaTemp = (aW*TW + aE*TE + aS*TS + aN*TN + b)/aP;

            celula->SetTemperatura(novaTemp*0.5 + celula->Temperatura()*0.5);
            celula->SetTemperatura(novaTemp);

            if (varreduraHorizontal) {
                ++i;
                if (i == limI){//int(lCelula.Dim(0))) {
                    i=iniI;
                    ++j;
                }
                if (j==limJ){//int(lCelula.Dim(1))) {
                    varreduraHorizontal = false;
                    //j=0 ;
                    //i=0;
                    i = iniJ;
                    j = iniI;
                    bProg.wait();
                    bVert.wait();
                }
            }
            else {
                ++j;
                if (j == limI){//int(lCelula.Dim(1))) {
                    //j=0;
                	j = iniI;
                    ++i;
                }
                if (i==limJ)//int(lCelula.Dim(0)))
                	varreduraCompleta = true;
            }
        }
    }
    {
    	boost::mutex::scoped_lock lock(m);
    	calorProAmbiente += localCalorProAmbiente;
    	tempSuperficialMedia += localTempSuperficialMedia;
    	areaTotal += localAreaTotal;
    }


//    	testectcr();



    //totTempoCorpoCTC += ( time(NULL) - tempoCorpoCTC );







}



void Segmento::testectcr()
{
	time_t tempoACV = time(NULL);
    AtualizaCoefVar();
    totTtempoACV += ( time(NULL) - tempoACV);

    time_t tempoCorpoCTC = time(NULL);

    areaTotal = 0.0;
    calorProAmbiente = 0.0; //calor total do segmento perdido para o ambiente
    tempSuperficialMedia = 0.0;

    //cout << "AKI#99";

    boost::mutex m;
    boost::barrier bHor(3);
    boost::barrier bVert(3);
    boost::barrier bProg(3);

    int k = lCelula.Dim(0) / 2;
    int limDim =  lCelula.Dim(0);

	boost::thread_group threadsCaps;



	threadsCaps.add_thread(new boost::thread(&Segmento::CalcTempCelAf, this, 0, 0, limDim, k, boost::ref(m), boost::ref(bHor), boost::ref(bVert), boost::ref(bProg )));
	threadsCaps.add_thread(new boost::thread(&Segmento::CalcTempCelAf, this, 0, k +1, limDim, limDim, boost::ref(m), boost::ref(bHor), boost::ref(bVert), boost::ref(bProg )));
	threadsCaps.add_thread(new boost::thread(&Segmento::CalcTempCelAfMid, this, 0, k, limDim, k, boost::ref(m), boost::ref(bHor), boost::ref(bVert), boost::ref(bProg )));

	threadsCaps.join_all();

    //CalcTempCelAf(0, 0, lCelula.Dim(0), lCelula.Dim(0), m, bHor, bVert, bProg);

    time_t tempoCTCT = time(NULL);
    CalculaTempCompTec();
    totTempoCTCT += (time(NULL) - tempoCTCT);

    tempSuperficialMedia /= areaTotal;

}

void Segmento::CalcTempCelSwt()
{
	if ( geometria == RETANGULAR)
		//testectcr();
		CalcTempCel();
	else
		CalcTempCel();
}
