//Classe: TransporteSangue
//Autor: Cyro Albuquerque Neto
//Data: 12/05/08

#include "transportesangue.h"

//---------------------------------------------------------------------------
TransporteSangue::TransporteSangue()
{
    hemoglobina=15.4;
    excessoBase=0.0;
    //PRECISAO=1.0e-6;
    temperatura=37.0;
    pH=7.4;
    saturacaoO2=0.98;
}

//---------------------------------------------------------------------------
/*TransporteSangue::TransporteSangue(double _hemoglobina, double _excessoBase=0.0) : 
    hemoglobina(_hemoglobina), 
    excessoBase(_excessoBase)
{
    PRECISAO=1.0e-6;
    temperatura=37.0;
}*/

//---------------------------------------------------------------------------
void TransporteSangue::CalcCoefSolub()
{
    coefSolub.SetO2(0.0000293*(1+log10(temperatura/37.0)+
                   0.00012*pow(temperatura-37.0,2.0))); //Thomas (1972)
    coefSolub.SetCO2(0.0307+0.00057*(37.0-temperatura)+
                   0.00002*pow((37.0-temperatura),2.0)); //Douglas (1988)
}

//---------------------------------------------------------------------------
double TransporteSangue::pK(double _pH)
{
    return 6.086+0.042*(7.4-_pH)+(38.0-temperatura)
           *(0.00472+0.00139*(7.4-_pH));
}

//---------------------------------------------------------------------------
void TransporteSangue::CalcpH(double pressaoCO2)
{
    double pH2 = 7.4, pH1 = pH2, HCO3_38;
    bool convergencia = false, limiteIteracao = false;
    const int MAXITERACAO = 10000;
    int iteracao = 0;

    double pCO2_38 = pressaoCO2*pow(10.0,0.021*(38.0-temperatura));

    while (!convergencia && !limiteIteracao) 
    {
        pH1 = (pH2+pH1)/2.0;
        HCO3_38 = 0.03015*pCO2_38*pow(10.0,0.958*pH1-5.775);
        pH2 = 7.4-1/(9.5+1.63*hemoglobina)*(HCO3_38-24.0-excessoBase/(1.0-0.0143*hemoglobina));
        if (fabs((pH2-pH1)/pH1) < PRECISAO) convergencia = true;
        if (iteracao >= MAXITERACAO)
        {
            limiteIteracao = true;
            cout << "Erro - limite da convergencia do pH atingido" << endl;
            cout << endl << "PCO2 " << pressaoCO2 << endl;
            exit(1);
        }
        iteracao++;
    }
    pH = pH2 + 0.0147*(38.0-temperatura) 
         + 0.003*hemoglobina*(1.0-saturacaoO2);
}

//---------------------------------------------------------------------------
void TransporteSangue::CalcSaturacaoO2(double pressaoO2)
{
    double N = pressaoO2*pow(10.0,(0.48*(pH-7.4) -0.024*(temperatura-37.0) 
               -0.0013*excessoBase));
    saturacaoO2 = (pow(N,4.0) - 15.0*pow(N,3.0) + 2045.0*pow(N,2.0) + 2000.0*N)/
         (pow(N,4.0) - 15.0*pow(N,3.0) + 2400.0*pow(N,2.0) - 31100.0*N + 2.4e6);
    if (saturacaoO2 < 0.0) saturacaoO2 = 0.0;
    else if (saturacaoO2 > 1.0) saturacaoO2 = 1.0;
}

//---------------------------------------------------------------------------
void TransporteSangue::CalcSatO2pH(TGas pressao)
{
    double satAux = 0.80; 
    saturacaoO2 = 0.80;

    bool convergencia = false, limiteIteracao = false;
    const int MAXITERACAO = 100000;
    int iteracao = 0;
    
    //Valor de pH inicial
    //Aqui deve existir possibilidade do inicial ser o anterior
    double pH_38 = 7.4; 
    double pHAntigo = pH_38; 
    double HCO3_38 = 0.0;

    //Corrige pCO2 para 38oC
    double pCO2_38 = pressao.CO2()*pow(10.0,0.021*(38.0-temperatura));

    //Calcula pH sem corrigir saturao de O2 nem temperatura
    while (!convergencia && !limiteIteracao) 
    {
        //Atualiza pH antigo
        pHAntigo = (pH_38+pHAntigo)/2.0;

        //Calcula [HCO3-] a 38oC, equao de Henderson-Hasselbalch
        HCO3_38 = 0.03015*pCO2_38*pow(10.0,0.958*pHAntigo-5.775);

        //Calcula novo pH, equao do BE
        pH_38 = 7.4-1/(9.5+1.63*hemoglobina)
             *(HCO3_38-24.0-excessoBase/(1.0-0.0143*hemoglobina));
        
        //Verifica convergncia
        if (fabs((pH_38-pHAntigo)/pHAntigo) < PRECISAO) convergencia = true;

        //Verifica se atingiu mximo de iteraes
        if (iteracao >= MAXITERACAO) {
            limiteIteracao = true;
            cout << "Erro1 - limite da convergencia do pH e Saturacao de O2 atingido" << endl;
            exit(1);
        }

        //Passo na iterao
        iteracao++;
    }

    //Calcula saturao de O2 e pH corrigido
    convergencia = false; 
    limiteIteracao = false;
    iteracao = 0;
    while (!convergencia && !limiteIteracao) 
    {
        //Atualiza pH antigo
        pHAntigo = 0.5*pH+0.5*pHAntigo; 

        //Corrige pH de 38oC para temperatura atual e saturao de O2
        pH = pH_38 + 0.0147*(38.0-temperatura) 
             + 0.003*hemoglobina*(1.0-saturacaoO2);

        //Atualiza saturao de O2 auxiliar
        satAux = 0.5*saturacaoO2+0.5*satAux; 

        //Calcula nova saturao de O2
        CalcSaturacaoO2(pressao.O2());

        //Verifica convergncia
        if (fabs((pH-pHAntigo)/pHAntigo) < PRECISAO 
            && fabs((saturacaoO2-satAux)/saturacaoO2) < PRECISAO) 
                convergencia = true;

        //Verifica se atingiu mximo de iteraes
        if (iteracao >= MAXITERACAO) {
            limiteIteracao = true;
            cout << "Erro2 - limite da convergencia do pH e Saturacao de O2 atingido" << endl;
            exit(1);
        }

        //Passo na iterao
        iteracao++;
    }
}
    
//---------------------------------------------------------------------------
double TransporteSangue::CalcConteudoO2(double pressaoO2)
{
    //cout << "SO2: " << saturacaoO2 << " pO2: " << pressaoO2 << endl;
    return 1.34*hemoglobina/100.0*saturacaoO2+coefSolub.O2()*pressaoO2;
}

//---------------------------------------------------------------------------
double TransporteSangue::CalcConteudoCO2(double pressaoCO2)
{
    conteudoCO2pl = coefSolub.CO2()*pressaoCO2*
                    (1.0+pow(10.0,pH-pK(pH)))*0.02226;
    return conteudoCO2pl*(1.0-0.0289*hemoglobina/
           ((3.352-0.456*saturacaoO2)*(8.142-pH)));
}

//---------------------------------------------------------------------------
TGas TransporteSangue::CalcConteudo(TGas pressao, double _temperatura)
{
    TGas conteudo;
    SetTemperatura(_temperatura);
    CalcCoefSolub();
    CalcSatO2pH(pressao);
    conteudo.SetO2(CalcConteudoO2(pressao.O2()));
    conteudo.SetCO2(CalcConteudoCO2(pressao.CO2()));
    return conteudo;
}

//---------------------------------------------------------------------------
double TransporteSangue::ConteudoCO2(double _saturacaoO2, double _pressaoCO2, double temp)
{
    temperatura = temp;
    saturacaoO2 = _saturacaoO2;
    CalcCoefSolub();
    CalcpH(_pressaoCO2);
    return CalcConteudoCO2(_pressaoCO2);
}

//-------------------------------------------------------------------------
//Calcular analiticamente
TGas TransporteSangue::CalcDerivada(TGas pressao, double temp)
{
    const double DPRESSAO = 0.5;
    TGas derivada, conteudo1, conteudo2, pressaoAux;
    SetTemperatura(temp);

    //O2
    pressaoAux.SetGas(pressao.O2()-DPRESSAO,pressao.CO2());
    conteudo1 = CalcConteudo(pressaoAux,temp);
    pressaoAux.SetGas(pressao.O2()+DPRESSAO,pressao.CO2());
    conteudo2 = CalcConteudo(pressaoAux,temp);
    derivada.SetO2((conteudo2.O2()-conteudo1.O2())/(2.0*DPRESSAO));

    //CO2
    pressaoAux.SetGas(pressao.O2(),pressao.CO2()-DPRESSAO);
    conteudo1 = CalcConteudo(pressaoAux,temp);
    pressaoAux.SetGas(pressao.O2(),pressao.CO2()+DPRESSAO);
    conteudo2 = CalcConteudo(pressaoAux,temp);
    derivada.SetCO2((conteudo2.CO2()-conteudo1.CO2())/(2.0*DPRESSAO));

    return derivada;
}

//-------------------------------------------------------------------------
//No deve ser usado no clculo - SEM compPulmao
TGas TransporteSangue::CalcPressao(TGas conteudo, double _temperatura)
{
    TGas pressao1, pressao2;
    bool convergencia = false, limiteIteracao = false;
    const int MAXITERACAO = 20000;
    int iteracao = 0;

    SetTemperatura(_temperatura);
    CalcCoefSolub();
    pressao1.SetGas(70.0,45.0);
    pressao2.SetGas(70.0,45.0);

    saturacaoO2 = 0.80;

    //Valor de pH inicial
    double pH_38 = 7.4; 
    double pHAntigo = pH_38; 
    double HCO3_38 = 0.0;


    while (!convergencia && !limiteIteracao) {
        pressao1.SetO2(pressao2.O2()*0.1+pressao1.O2()*0.9);
        pressao1.SetCO2(pressao2.CO2()*0.1+pressao1.CO2()*0.9);
        //CalcSatO2pH(pressao1);

        //pH e sO2
        //Atualiza pH antigo
        pHAntigo = (pH_38+pHAntigo)/2.0;

        //Corrige pCO2 para 38oC
        double pCO2_38 = pressao1.CO2()*pow(10.0,0.021*(38.0-temperatura));

        //Calcula [HCO3-] a 38oC, equao de Henderson-Hasselbalch
        HCO3_38 = 0.03015*pCO2_38*pow(10.0,0.958*pHAntigo-5.775);

        //Calcula novo pH, equao do BE
        pH_38 = 7.4-1/(9.5+1.63*hemoglobina)
             *(HCO3_38-24.0-excessoBase/(1.0-0.0143*hemoglobina));

        //Corrige pH de 38oC para temperatura atual e saturao de O2
        pH = pH_38 + 0.0147*(38.0-temperatura) 
             + 0.003*hemoglobina*(1.0-saturacaoO2);

        //Calcula nova saturao de O2
        CalcSaturacaoO2(pressao1.O2());

        //CO2
        conteudoCO2pl = conteudo.CO2()/(1.0-0.0289*hemoglobina/
                        ((3.352-0.456*saturacaoO2)*(8.142-pH)));
        pressao2.SetCO2(conteudoCO2pl/(coefSolub.CO2()*(1.0+pow(10.0,pH-pK(pH)))*0.02226));
        //O2
        pressao2.SetO2(conteudo.O2()/(1.34*hemoglobina/100.0*
                      saturacaoO2/pressao1.O2()+coefSolub.O2()));
        if (fabs((pressao2.O2()-pressao1.O2())/pressao1.O2()) < PRECISAO && 
            fabs((pressao2.CO2()-pressao1.CO2())/pressao1.CO2()) < PRECISAO) {
            convergencia = true;
            break;
        }
        if (iteracao >= MAXITERACAO) {
            limiteIteracao = true;
            cout << "Erro - limite da convergencia do calculo das pressoes parciais atingido" << endl;
            exit(1);
        }
        iteracao++;
    } 
    //cout << "iteracao pressao: " << iteracao << endl;
    return pressao2;
}
