//Cabecalho da classe TransporteSangue
//Autor: Cyro Albuquerque Neto
//Data: 12/05/2008

#ifndef TRANSPORTESANGUEH
#define TRANSPORTESANGUEH

#include <cmath>
#include <iostream>
#include <cstdlib>

#include "definicao.h"

using namespace std;

typedef Gas<double> TGas;

class TransporteSangue
{
public:

    TransporteSangue();
    //TransporteSangue(double _hemoglobina, double _excessoBase);

    void   SetTemperatura(double numero) { temperatura = numero; }
    double Temperatura() const { return temperatura; }
    void   SetHemoglobina(double numero) { hemoglobina = numero; }
    double Hemoglobina() const {return hemoglobina;};
    void   SetExcessoBase(double numero) {excessoBase = numero;};
    double ExcessoBase() const {return excessoBase;};
    double ConteudoCO2Plasma() const {return conteudoCO2pl;};
    void   SetPH(double numero) {pH = numero;};
    double PH() const { return pH; }
    double SaturacaoO2() const { return saturacaoO2; }


    //Esses que devem ser usados na simulao

    //Calcular concentrao (em mol/m3),
    //com entrada da presso parcial (em Pa)
    TGas CalcConcentracao(TGas pres, double temp) {
        TGas conc;
        pres.DividePor(133.32237);              //de Pa para mmHg
        //cout << pres.O2() << "=" << pres.CO2() << " ";
        conc = CalcConteudo(pres,temp);
        //cout << conc.O2() << "%" << conc.CO2() << " ";
        conc.SetO2(conc.O2()*1000.0/22.4);      //de mlO2/ml para mol/m3
        conc.SetCO2(conc.CO2()*1000.0/22.26);   //de mlCO2/ml para mol/m3
        return conc;
    }

    //Sem compPulmao
    //Calcular presso parcial (em Pa),
    //com entrada da concentrao (em mol/m3)
    TGas CalcPressaoParcial(TGas conc, double temp) {
        TGas pres;
        conc.SetO2(conc.O2()*22.4e-3);      //de mol/m3 para mlO2/ml
        conc.SetCO2(conc.CO2()*22.26e-3);   //de mol/m3 para mlCO2/ml
        //cout << conc.O2() << " " << conc.CO2() << " ";
        pres = CalcPressao(conc,temp);
        //cout << pres.O2() << " " << pres.CO2() << " ";
        pres.MultiplicaPor(133.32237);      //de mmHg para Pa
        return pres;
    }

    // Derivada da concentrao em relao a presso parcial
    TGas CalcDerivadaConcentracao(TGas pres, double temp) {
        TGas der;
        pres.DividePor(133.32237);              //de Pa para mmHg
        der = CalcDerivada(pres,temp);
        der.SetO2(der.O2()*1000.0/22.4/133.32237);      //de mlO2/(ml.mmHg) para mol/(m3.Pa)
        der.SetCO2(der.CO2()*1000.0/22.26/133.32237);   //de mlCO2/(ml.mmHg) para mol/(m3.Pa)
        return der;
    }
        


private:

    double hemoglobina;
    double excessoBase;

    double pH; 
    double saturacaoO2;
    double temperatura;
    TGas   coefSolub;
    double conteudoCO2pl;

    TGas   CalcConteudo(TGas pressao, double _temperatura=37.0);
    TGas   CalcPressao(TGas conteudo, double _temperatura=37.0); 
    TGas   CalcDerivada(TGas pressao, double _temperatura=37.0);
    double ConteudoCO2(double _saturacaoO2, 
                       double _pressaoCO2, 
                       double temp=37.0);
    void   CalcSaturacaoO2(double pressaoO2);
    void   CalcSatO2pH(TGas pressao);

    void   CalcCoefSolub();
    double pK(double _pH);
    void   CalcpH(double pressaoCO2);
    double CalcConteudoO2(double pressaoO2);
    double CalcConteudoCO2(double pressaoCO2);
};

#endif
